#ifndef REGOBSERVER_H
#define REGOBSERVER_H

#include <fstream>

#include <QThread>

#include "itkCommand.h"
#include "itkVersorRigid3DTransformOptimizer.h"

#include "PetSpectFusionViewer.h"
#include "typedefs.h"

class QRegistrationThread;

 /**
 * \brief This class is responsible for keeping track of the registration process
 *
 * \authors Brian Jensen
 *          <br>
 *          {jensen}\@cs.tum.edu
 * \ingroup PetSpectFusion
 * \version 1.0
 * \date 16.12.2007
 *
 * \par License:
 * Copyright (c) 2007,
 * This programm was created as part of a student research project in cooperation
 * with the Department for Computer Science, Chair XVI
 * and the Nuklearmedizinische Klinik, Klinikum Rechts der Isar
 *
 * <br>
 * <br>
 * All rights reserved.
 * <br>
 * <br>
 * See <a href="COPYRIGHT.txt">COPYRIGHT.txt</a> for details.
 * <br>
 * <br>
 * This software is distributed WITHOUT ANY WARRANTY; without even 
 * <br>
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * <br>
 * PURPOSE.  See the <a href="COPYRIGHT.txt">COPYRIGHT.txt</a> notice
 * for more information.
 *
 */
class CommandIterationUpdate : public itk::Command 
{
public:
	typedef  CommandIterationUpdate   Self;
	typedef  itk::Command             Superclass;
	typedef itk::SmartPointer<Self>  Pointer;
	itkNewMacro( Self );
	
	typedef itk::VersorRigid3DTransformOptimizer     OptimizerType;
	typedef const OptimizerType *  ConstOptimizerPointer;
	typedef OptimizerType * OptimizerPointer;

private:
	//used for saving data into a gnuplot file
	std::ofstream transFile, anglesFile, metricFile;
	std::string plotFileName;
	bool plotData, stopReg;
	
	//used for updating the gui if present
	QRegistrationThread* regThread;

protected:
	CommandIterationUpdate()
	{ plotData = false;  regThread = NULL; stopReg = false;};

public:
	void closePlotFiles();

	void Execute(itk::Object *caller, const itk::EventObject & event);

	void Execute(const itk::Object * object, const itk::EventObject & event);
	
	void enablePlotting(const char* fileName);
	
	void setDisplayThread(QRegistrationThread* thread);
	
	void stopRegistration();
	
};


	//This class is used by the multi resolution registration
template <typename TRegistration>
class RegistrationInterfaceCommand : public itk::Command 
{
	public:
		typedef  RegistrationInterfaceCommand   Self;
		typedef  itk::Command                   Superclass;
		typedef  itk::SmartPointer<Self>        Pointer;
		itkNewMacro( Self );
	protected:
		RegistrationInterfaceCommand() {};

	private:

	public:
		typedef   TRegistration RegistrationType;
		typedef   RegistrationType * RegistrationPointer;
		typedef   itk::VersorRigid3DTransformOptimizer OptimizerType;
		typedef   OptimizerType * OptimizerPointer;

	void Execute(itk::Object * object, const itk::EventObject & event)
	{
		if( !(itk::IterationEvent().CheckEvent( &event )) )
		{
			return;
		}

		RegistrationPointer registration =
                            dynamic_cast<RegistrationPointer>( object );
	
		OptimizerPointer optimizer = dynamic_cast< OptimizerPointer >( 
                       registration->GetOptimizer() );

		std::cout << "Current level: " << registration->GetCurrentLevel() << std::endl;

	}
	
	void Execute(const itk::Object * , const itk::EventObject & )
	{
		return;
	}
	
};


class QPetSpectFusionViewer;

 /**
 * \brief This class contains the worker thread that Viewer App uses to do the registration
 *
 * \authors Brian Jensen
 *          <br>
 *          {jensen}\@cs.tum.edu
 * \ingroup PetSpectFusion
 * \version 1.0
 * \date 16.12.2007
 *
 * \par License:
 * Copyright (c) 2007,
 * This programm was created as part of a student research project in cooperation
 * with the Department for Computer Science, Chair XVI
 * and the Nuklearmedizinische Klinik, Klinikum Rechts der Isar
 *
 * <br>
 * <br>
 * All rights reserved.
 * <br>
 * <br>
 * See <a href="COPYRIGHT.txt">COPYRIGHT.txt</a> for details.
 * <br>
 * <br>
 * This software is distributed WITHOUT ANY WARRANTY; without even 
 * <br>
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * <br>
 * PURPOSE.  See the <a href="COPYRIGHT.txt">COPYRIGHT.txt</a> notice
 * for more information.
 *
 */
class QRegistrationThread : public QThread
{
	Q_OBJECT
	
	public:
		QRegistrationThread(QObject* parent, QPetSpectFusionViewer* viewer, InternalImageType::Pointer fImage, InternalImageType::Pointer mImage,
		int numBins, double minStep, double maxStep, float samples, int maxIter, OptimizerScalesType scales, ParametersType tParams = NULL);
		void registrationUpdate(ParametersType params, double metricVal, int interation);
		void stopRegistration();
		void enablePlotting(std::string fileName);
		void setMultiRes(bool set, int lev);
		
	signals:
		void parametersUpdated(ParametersType params, double metricVal, int interation);
		
	protected:
		void run();
	
	private:
		QPetSpectFusionViewer* display;
		InternalImageType::Pointer fixedImage;
		InternalImageType::Pointer movingImage;
		ParametersType params;
		CommandIterationUpdate::Pointer observer;
		
		int bins, levels, maxIterations;
		float sampleRate;
		double minSize, maxSize;
		bool multiRes;
		std::string plotName;
		
		OptimizerScalesType optimizerScales;
		
};

#endif
