#ifndef REGISTRATION_VIEW_H
#define REGISTRATION_VIEW_H

#include <QtGui>
#include <vector>


#include "typedefs.h"
#include "ViewObserver.h"

class QPetSpectFusionViewer;

//Valid color tables offered by this widget
enum ColorTable { HotRed, HotBlue, LinearRed, BlackWhite };
extern const char *ColorTableNames[];
const char ColorTableCount = 4;

 /**
 *	 $Id$
 *   $Lastlog$
 *
 * \brief This class contains display widget for showing the slice views.  
 *
 * \authors Brian Jensen
 *          <br>
 *          {jensen}\@cs.tum.edu
 * \ingroup PetSpectFusion
 * \version 1.20
 * \date 11.08.2008
 *
 * \par License:
 * Copyright (c) 2007 - 2009,
 * This program was started as part of a student research project in cooperation
 * with the Department for Computer Science, Chair XVI
 * and the Nuklearmedizinische Klinik, Klinikum Rechts der Isar
 *
 */
class QRegistrationView : public QWidget
{
	Q_OBJECT

	public:
		QRegistrationView(QPetSpectFusionViewer* viewer = NULL);
		
		InternalImageType::Pointer getFixedImage();
		InternalImageType::Pointer getMovingImage();
		void setFixedImage(InternalImageType::Pointer image);
		void setMovingImage(InternalImageType::Pointer image);
		
		void setTransformParams(ParametersType params);
		void setZoomFactor(int factor);
		void setInterpolationMode(int mode);
		void showCheckeredImage(bool show);
		
		//functions for getting / setting the color tables
		ColorTable getFixedImageColorTable();
		ColorTable getMovingImageColorTable();
		void setFixedImageColorTable(ColorTable table);
		void setMovingImageColorTable(ColorTable table);
	
		//windowing functions
		float getFixedWindowMin();
		void setFixedWindowMin(float val);
		float getFixedWindowMax();
		void setFixedWindowMax(float val);
		float getMovingWindowMin();
		void setMovingWindowMin(float val);
		float getMovingWindowMax();
		void setMovingWindowMax(float val);		
		bool isFixedWindowMaxMode();
		void setFixedWindowMaxMode(bool setMaxMode);
		bool isMovingWindowMaxMode();
		void setMovingWindowMaxMode(bool setMaxMode);
		InternalPixelType getFixedImageMin();
		InternalPixelType getFixedImageMax();
		InternalPixelType getMovingImageMin();
		InternalPixelType getMovingImageMax();
	
		//window region functions
		void setFixedWindowRegion(RegionType region);
		void calculateFixedWindowRegion();
		void calculateMovingWindowRegion();
		RegionType getFixedWindowRegion();
		void setWindowingMode(bool regionMode);
		bool getWindowingMode();
		void showWindowRegion(bool show);
	
		//checkerboard 
		void setCheckerBoardOrder(bool fixedFirst);
		bool getCheckerBoardOrder();
		CheckerBoardType::PatternArrayType getCheckerBoardPattern();
		void setCheckerBoardPattern(CheckerBoardType::PatternArrayType pattern);
	
		//VOI types
		void setVoiRegion(RegionType region);
		RegionType getVoiRegion();
		bool isShowingVoi();
		void showVoi(bool show);
	
		//ViewObserver functions
		void addObserver(ViewObserver* observer);
		void removeObserver(ViewObserver* observer);
	
	public slots:
		void transSliceChanged(int sliceNr);
		void corSliceChanged(int sliceNr);
		void sagSliceChanged(int sliceNr);
		void alphaChanged(int val);
		void updateView();

	private:
		//The slice views supported
		enum SliceView { TransSlice, CorSlice, SagSlice };	
		void updateSliceView(SliceView view);
	
		InternalImageType::Pointer fixedImage;
		InternalImageType::Pointer movingImage;
		InternalImageType::SizeType fixedSize;
		InternalImageType::SpacingType fixedSpacing;
		
		TransformType::Pointer transform;
		
		//buffers for the various slice images		
		unsigned char* fixed_trans_slice_buffer;
		unsigned char* moving_trans_slice_buffer;
		unsigned char* fixed_cor_slice_buffer;
		unsigned char* moving_cor_slice_buffer;
		unsigned char* fixed_sag_slice_buffer;
		unsigned char* moving_sag_slice_buffer;
		
		unsigned int maxTransSlices;
		unsigned int maxCorSlices;
		unsigned int maxSagSlices;
		unsigned int curTransSlice;
		unsigned int curCorSlice;
		unsigned int curSagSlice;
		unsigned int alpha;
		unsigned int zoomFactor;
		unsigned int interpolationMode;
	
		QLabel* transSliceLabel;
		QLabel* corSliceLabel;
		QLabel* sagSliceLabel;
		QLabel* alphaLabel;
	
		//keep track of the color table in use internally
		ColorTable fixedImageColorTable;
		ColorTable movingImageColorTable;
	
		void setBlackWhiteColorTable(QImage *image);
		void setRedColorTable(QImage *image);
		void setHotRedColorTable(QImage *image);
		void setBlueWhiteColorTable(QImage *image);
	
		bool showCheckered;

		QSlider *alphaSlider;
		QSlider *transSliceSlider;
		QSlider *corSliceSlider;
		QSlider *sagSliceSlider;

		//used to display the slice pixmaps
		QLabel *transViewLabel;
		QLabel *corViewLabel;
		QLabel *sagViewLabel;

		QImage *transFixedImage;
		QImage *corFixedImage;
		QImage *sagFixedImage;
		QImage *transMovingImage;
		QImage *corMovingImage;
		QImage *sagMovingImage;
	
		//needed so only one thread at a time draws to the widget
		QMutex displayMutex;

		//needed to set the initial parameters of the display
		QPetSpectFusionViewer* display;
	
		//Parameters for the image windowing
		InternalPixelType fixedImageMin;
		InternalPixelType fixedImageMax;
		InternalPixelType movingImageMin;
		InternalPixelType movingImageMax;
	
		//windowing parameters (in percent)
		float winMaxFixed;
		float winMinFixed;
		float winMaxMoving;
		float winMinMoving;	
	
		//determines if window parameters are taken relative to the max intensity value, or the min / maxth median voxel intensity value 
		bool winMaxModeFixed;
		bool winMaxModeMoving;
	
		//VOI registration types
		RegionType voiRegion;
		bool showRegistrationVoi;
	
		//region windowing
		RegionType fixedWindowRegion;
		bool useWindowRegion;
		bool showWinRegion;
	
		//checkerboard image view order
		bool checkerFixedFirst;
		
		//checkerboard pattern
		CheckerBoardType::PatternArrayType checkerPattern;
	
		//list of obervers for this view
		std::vector<ViewObserver *> m_observers;
	
};


#endif