//
//  $Id$
//  $Lastlog$
//
//  Main.cxx
//  PetSpectFusion
//
//	Created 17/11/2009, The pseudo main of the application, calls AppMain upon entry
//  Copyright (c) 2007 - 2010 Brian Jensen. All rights reserved.
//

#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <QApplication>

#include "ImageIO.h"
#include "Log.h"
#include "PetSpectFusionApp.h"
#include "PetSpectFusionViewer.h"
#include "RegObserver.h"
#include "Registration.h"
#include "typedefs.h"

using namespace logging;
using namespace PetSpectFusionApp;

int main(int argc, char** argv)
{
	PetSpectFusionApp::init();
		
	int retVal = processCommandLine(argc, argv);
	//check for an error parsing command line options
	if(retVal)
		return retVal;
		
	//redirect the standard out and error if logging is wished
	if(logFileName != "")
	{
		enableFileLogging(logFileName);
	}
	
	logMessage("PetSpectFusion started");
	
	//transform only mode was wished, so perform the transform and exit
	if(transform_mode)
	{		
		int result;
		
		if(fixedFileName != "" && movingFileName != "" && outputFileName != "" && transform.IsNotNull())
		{
			std::ostringstream ostring(std::ostringstream::out);
			logMessage("Program is running in transform only mode");
			ostring << "transform params: " << transformParams;
			logMessage(ostring);
			ostring.str("");
			
			ostring << "fixed name: " << fixedFileName;
			logMessage(ostring);
			ostring.str("");
			ostring << "moving name: " << movingFileName;
			logMessage(ostring);
			ostring.str("");
			ostring << "output name: " << outputFileName;
			logMessage(ostring);
			ostring.str("");
			
			InternalImageType::Pointer fixedImage = loadMhdImage(fixedFileName.c_str());
			InternalImageType::Pointer movingImage = loadMhdImage(movingFileName.c_str());
			
			//make sure both images loaded properly
			if(fixedImage.IsNotNull() && movingImage.IsNotNull())
			{
				initializeTransform(transform, fixedImage, movingImage);
				transform->SetParameters(transformParams);
				
				logMessage("performing the transform...");
				
				typedef itk::ResampleImageFilter<InternalImageType, OutputImageType> ResampleFilterType;
				ResampleFilterType::Pointer resampler = ResampleFilterType::New();
				
				resampler->SetTransform(transform);
				resampler->SetInput(movingImage);
				resampler->SetSize(fixedImage->GetLargestPossibleRegion().GetSize());
				resampler->SetOutputOrigin( fixedImage->GetOrigin());
				resampler->SetOutputSpacing( fixedImage->GetSpacing());
				resampler->SetDefaultPixelValue(0);
				resampler->Update();
				
				writeOutputImageMhd(outputFileName.c_str(), resampler->GetOutput());
				
				result = 0;
			}
			
		}
		else
		{
			logError("Error: you must specify the fixed image, moving image, the output image, and a transform!");
			logError("For command usage type -h");
			result = 1;
		}
		
		//restore the original streams			
		if(fileLoggingEnabled()){
			disableFileLogging();
		}
		
		return result;
		
	}
	
	//either no options specified or graphics mode wished, so launch the GUI
	if(argc == 1 || graphics_mode)
	{		
		//need in case no parameters were specified on the command line
		graphics_mode = true;
		
		Q_INIT_RESOURCE(PetSpectFusion);
		
		QApplication app(argc, argv);
		
		QPetSpectFusionViewer viewer(fixedFileName, movingFileName, outputFileName, bins, 
									 sampleRate, minStepSize, maxStepSize, maxIterations, optimizerScales, zoomFactor);
		
		mainViewer = &viewer;
		
		if(doMultiRes)
			viewer.setMultiRes(levels);
		
		if(plotFileName != "")
			viewer.setPlotFile(plotFileName);
		
		
		if(transform.IsNotNull())
		{
			logMessage("a transform was set from the command line, setting it in display");
			viewer.setParameters(transformParams);
		}
		
		if(automatic_mode)
		{
			if(fixedFileName != "" && movingFileName != "" && outputFileName != "")
				viewer.doAutomaticReg();
			else
			{
				std::ostringstream ostring(std::ostringstream::out);
				
				logError("Error: you must specify both input and the output file names for an automatic registration!!");
				ostring << "fixed name: " << fixedFileName;
				logError(ostring);
				ostring.str("");
				ostring << "moving name: " << movingFileName;
				logError(ostring);
				ostring.str("");
				ostring << "output name: " << outputFileName;
				logError(ostring);
				ostring.str("");
				return 2;
			}
		}
		
		viewer.move(120, 20);
		viewer.show();
		int result =  app.exec();
		
		//restore the original streams
		if(fileLoggingEnabled()){
			disableFileLogging();
		}
		
		return result;
	}
	
	if( fixedFileName == "" || movingFileName == "" || outputFileName == "")
	{
		logError("Either the fixed, moving or output file names was not set! You must set all three!");
		return 3;
	}
	
	//start by loading the images
	InternalImageType::Pointer fixedImage = loadMhdImage(fixedFileName.c_str());
	InternalImageType::Pointer movingImage = loadMhdImage(movingFileName.c_str());
	
	if(fixedImage.IsNull() || movingImage.IsNull())
	{
		//Turn off file logging
		if(fileLoggingEnabled()){
			disableFileLogging();
		}
		
		return 4;
	}
	
	std::ostringstream ostring(std::ostringstream::out);
	
	ostring << "Registration started" << std::endl;
	ostring << "fixed image file: " << fixedFileName << std::endl;
	ostring << "moving image file: " << movingFileName << std::endl;
	ostring << "output image file: " << outputFileName;
	logMessage(ostring);
	ostring.str("");
	
	// Create the Command observer and register it with the optimizer.
	CommandIterationUpdate::Pointer observer = CommandIterationUpdate::New();
	RegistrationResultType result;
	
	if(plotFileName != "")
	{
		observer->enablePlotting(plotFileName.c_str());
	}
	
	if(doMultiRes)
	{

			//if a pure VOI registration was requested, avoid performing a full one
		if(!doVOIRegistration) 
		{
			result = doMattesMultiRegistration(fixedImage, movingImage, observer, bins,
											   sampleRate , minStepSize, maxStepSize, maxIterations, optimizerScales, levels);
		}
		
		//perform a VOI registration
		if(doVOIRegistration || doVOIAndFullRegistration)
		{
			
			typedef itk::ExtractImageFilter<InternalImageType, InternalImageType> RegionFilterType;
			RegionFilterType::Pointer regionFilter = RegionFilterType::New();
			regionFilter->SetInput(fixedImage);
			regionFilter->SetExtractionRegion(voiRegion);
		
			InternalImageType::Pointer voiImage = regionFilter->GetOutput();
			voiImage->Update();
			
			//use the parameters from the full registration as initial params for VOI registration
			if(doVOIAndFullRegistration) 
			{
				//zero out the rotation, VOI mode doesn't seem to handle it well
				result.params.SetElement(0, 0);
				result.params.SetElement(1, 0);
				result.params.SetElement(2, 0);
				result = doMattesMultiRegistration(voiImage, movingImage, observer, bins,
												   sampleRate , minStepSize, maxStepSize, maxIterations, optimizerScales, levels, result.params);
			}
			else 
			{
				result = doMattesMultiRegistration(voiImage, movingImage, observer, bins,
												   sampleRate , minStepSize, maxStepSize, maxIterations, optimizerScales, levels);
			}

		}
	}
	else
	{
		if(!doVOIRegistration)
		{
			result = doMattesRegistration(fixedImage, movingImage, observer, bins,
										  sampleRate, minStepSize, maxStepSize, maxIterations, optimizerScales);
		}
		//perform a VOI registration
		if(doVOIRegistration || doVOIAndFullRegistration)
		{
			
			typedef itk::ExtractImageFilter<InternalImageType, InternalImageType> RegionFilterType;
			RegionFilterType::Pointer regionFilter = RegionFilterType::New();
			regionFilter->SetInput(fixedImage);
			regionFilter->SetExtractionRegion(voiRegion);
			
			InternalImageType::Pointer voiImage = regionFilter->GetOutput();
			voiImage->Update();
			
			//use the parameters from the full registration as initial params for VOI registration
			if(doVOIAndFullRegistration) 
			{
				//zero out the rotation, VOI mode doesn't seem to handle it well
				result.params.SetElement(0, 0);
				result.params.SetElement(1, 0);
				result.params.SetElement(2, 0);
				result = doMattesRegistration(voiImage, movingImage, observer, bins,
												   sampleRate , minStepSize, maxStepSize, maxIterations, optimizerScales, result.params);
			}
			else 
			{
				result = doMattesRegistration(voiImage, movingImage, observer, bins,
												   sampleRate , minStepSize, maxStepSize, maxIterations, optimizerScales);
			}
		}
	}
	
	if(plotFileName != "")
	{
		//close plot files, perhaps use a destructor later..?
		observer->closePlotFiles();
	}	
	
	if (!result.error) 
	{
		logMessage("Registration completed!");
		writeRegisteredImage(outputFileName.c_str(), fixedImage, movingImage, result.params);
	}
	
	//Turn off file logging if it was activated
	if(fileLoggingEnabled()){
		disableFileLogging();
	}
	
	return result.error;
	
}
