//
//  $Id$
//  $Lastlog$
//
//  PetSpectFusionViewer.cxx
//  PetSpectFusion
//
//	Created 16/12/2007, Base class for the GUI, contains the main window etc..
//  Copyright (c) 2007 - 2009 Brian Jensen. All rights reserved.
//

#include <QtGui>

#include "typedefs.h"
#include "CheckerboardViewDialog.h"
#include "ImageIO.h"
#include "Log.h"
#include "PetSpectFusionApp.h"
#include "PetSpectFusionViewer.h"
#include "Registration.h"
#include "RegObserver.h"
#include "RegSettingsDialog.h"
#include "RegistrationViewDialog.h"

#include "time.h"

QPetSpectFusionViewer::QPetSpectFusionViewer(std::string fFileName, std::string mFileName, std::string oFileName, int nrBins, 
				float sRate, float minStep, float maxStep, int maxIter, OptimizerScalesType scales, int zoomFactor)
{	
	regIsRunning = false;
	showCheckeredImg = false;
	automatic_mode = false;
	
	updateAngles = true;
	
	fixedFileName = fFileName;
	movingFileName = mFileName;
	outputFileName = oFileName;
	
	optimizerScales = scales;
	ParametersType t_params(6);
	transformParams = t_params;
	
	bins = nrBins;
	sampleRate = sRate;
	minStepSize = minStep;
	maxStepSize = maxStep;
	maxIterations = maxIter;

	plotFileName = "";
	plotting = false;
	levels = DEFAULT_MULTIRES_LEVELS;
	
	createComponents();
	createActions();
	createMenus();

	regView->setZoomFactor(zoomFactor);
	switch (zoomFactor) {
		case 1:
		default:
			zoom1Act->setChecked(true);
			break;
		case 2:
			zoom2Act->setChecked(true);
			break;
		case 3:
			zoom3Act->setChecked(true);
			break;
		case 4:
			zoom4Act->setChecked(true);
			break;
		case 6:
			zoom6Act->setChecked(true);
			break;
	}
	
	if(fixedFileName != "")
	{
		fixedImage = loadMhdImage(fixedFileName.c_str());
		if(fixedImage.IsNotNull())
			regView->setFixedImage(fixedImage);
		
		//if fixed window region is valid, pass it to the registration view
		if(PetSpectFusionApp::fixedWindowRegion.GetSize(0) != 0 && 
		   PetSpectFusionApp::fixedWindowRegion.GetSize(1) != 0 &&
		   PetSpectFusionApp::fixedWindowRegion.GetSize(2) != 0)
		{
			regView->setFixedWindowRegion(PetSpectFusionApp::fixedWindowRegion);
			regView->setWindowingMode(true);
		}
		
		//only use the VOI region if it is valid
		if(PetSpectFusionApp::voiRegion.GetSize(0) != 0 &&
		   PetSpectFusionApp::voiRegion.GetSize(1) != 0 &&
		   PetSpectFusionApp::voiRegion.GetSize(2) != 0)
		{
			voiRegion = PetSpectFusionApp::voiRegion;
			if(PetSpectFusionApp::doVOIRegistration)
				voiRegistrationAct->setChecked(true);
			else if(PetSpectFusionApp::doVOIAndFullRegistration)
				voiAndFullRegistrationAct->setChecked(true);
			
		}
	}
	
	if(movingFileName != "")
	{
		movingImage = loadMhdImage(movingFileName.c_str());
		if(movingImage.IsNotNull())
			regView->setMovingImage(movingImage);
	}
	
	regView->updateView();
	
	multiResAct->setChecked(DEFAULT_DO_MULTIRES);
		
	std::string titleStr("PetSpectFusion 1.31");
	regViewDialogOpen = false;
	checkerboardViewDialogOpen = false;
	regSettingsDialogOpen = false;
	isVoiAndFullFirstPass = false;
	
	setWindowTitle(tr(titleStr.c_str()));
	
}

/**
 *	Function catches the window close event and pops up a save dialog
 *
 */
void QPetSpectFusionViewer::closeEvent(QCloseEvent *event)
{
		//we assume that because we are in automatic mode, the output file name was specified and registration performed
	if(automatic_mode)
	{
		writeRegisteredImage(outputFileName.c_str(), fixedImage, movingImage, transformParams);
	}
	else if(fixedImage && movingImage)
	{
		QMessageBox::StandardButton choice = QMessageBox::question(this, tr("Save before exiting"), 
			tr("Save Registered Image before exiting"), QMessageBox::Yes | QMessageBox::No, QMessageBox::No);
		
		if(choice == QMessageBox::Yes)
			saveRegisteredImg();
	
	}

	event->accept();
}

void QPetSpectFusionViewer::createActions()
{
	//file menu actions
	openFixedImageAct = new QAction(tr("Open &Fixed Image"), this);
	openFixedImageAct->setShortcut(tr("Ctrl+F"));
	openFixedImageAct->setStatusTip(tr("Open the fixed image file"));
	connect(openFixedImageAct, SIGNAL(triggered()), this, SLOT(openFixedImage()));
	
	openMovingImageAct = new QAction(tr("Open &Moving Image"), this);
	openMovingImageAct->setShortcut(tr("Ctrl+M"));
	openMovingImageAct->setStatusTip(tr("Open the moving image file"));
	connect(openMovingImageAct, SIGNAL(triggered()), this, SLOT(openMovingImage()));

	//save submenu
	saveAllAct = new QAction(tr("Save &all"), this);
	saveAllAct->setShortcut(tr("Ctrl+A"));
	saveAllAct->setStatusTip(tr("Save all the registration data"));
	connect(saveAllAct, SIGNAL(triggered()), this, SLOT(saveAll()));
	
	saveRegisteredAct = new QAction(tr("&Save registered"), this);
	saveRegisteredAct->setShortcut(tr("Ctrl+S"));
	saveRegisteredAct->setStatusTip(tr("Saves the newly registered moving image"));
	connect(saveRegisteredAct, SIGNAL(triggered()), this, SLOT(saveRegisteredImg()));

	saveCheckDiffAct = new QAction(tr("Save ch&eckerboard"), this);
	saveCheckDiffAct->setShortcut(tr("Ctrl+E"));
	saveCheckDiffAct->setStatusTip(tr("Saves a checkerboard composite of the registered and fixed images"));
	connect(saveCheckDiffAct, SIGNAL(triggered()), this, SLOT(saveCheckDiffImg()));

	saveAbsDiffAct = new QAction(tr("Save abs. diff"), this);
	saveAbsDiffAct->setStatusTip(tr("Save absolute difference Image"));
	connect(saveAbsDiffAct, SIGNAL(triggered()), this, SLOT(saveAbsDiffImg()));
	
	saveTransformParamsAct = new QAction(tr("Save &parameters"), this);
	saveTransformParamsAct->setShortcut(tr("Ctrl+P"));
	saveTransformParamsAct->setStatusTip(tr("Save the transform parameter results"));
	connect(saveTransformParamsAct, SIGNAL(triggered()), this, SLOT(saveTransformParams()));
	
	exitAct = new QAction(tr("E&xit"), this);
	exitAct->setShortcut(tr("Ctrl+Q"));
	exitAct->setStatusTip(tr("Exit the program"));
	connect(exitAct, SIGNAL(triggered()), this, SLOT(close()));

	//registration menu actions
	doRegAct = new QAction(tr("Perform registration"), this);
	doRegAct->setStatusTip(tr("Perform registration"));
	connect(doRegAct, SIGNAL(triggered()), this, SLOT(doReg()));
	
	multiResAct = new QAction(tr("Use Multi Resolution"), this);
	multiResAct->setStatusTip(tr("Perform registration using multi resolution image pyramids"));
	multiResAct->setShortcut(tr("Ctrl+U"));
	multiResAct->setCheckable(true);
	multiResAct->setChecked(false);
	
	fullRegistrationAct = new QAction(tr("Full& Registrion"), this);
	fullRegistrationAct->setStatusTip(tr("Perform registration using the full domain from both images"));
	fullRegistrationAct->setShortcut(tr("Ctrl+L"));
	fullRegistrationAct->setCheckable(true);
	fullRegistrationAct->setChecked(true);
	connect(fullRegistrationAct, SIGNAL(triggered()), this, SLOT(setFullMode()));
	
	voiRegistrationAct = new QAction(tr("VO&I Registration"), this);
	voiRegistrationAct->setStatusTip(tr("Perform registration using a VOI from the fixed image"));
	voiRegistrationAct->setShortcut(tr("Ctrl+I"));
	voiRegistrationAct->setCheckable(true);
	voiRegistrationAct->setChecked(false);
	connect(voiRegistrationAct, SIGNAL(triggered()), this, SLOT(setVoiMode()));
	
	voiAndFullRegistrationAct = new QAction(tr("Full and VOI registration"), this);
	voiAndFullRegistrationAct->setStatusTip(tr("Perform a full registration then a VOI registration"));
	voiAndFullRegistrationAct->setShortcut(tr("Ctrl+B"));
	voiAndFullRegistrationAct->setCheckable(true);
	voiAndFullRegistrationAct->setChecked(false);
	connect(voiAndFullRegistrationAct, SIGNAL(triggered()), this, SLOT(setVoiAndFullMode()));
	
	regSettingsAct = new QAction(tr("Options"), this);
	regSettingsAct->setStatusTip(tr("Change the advanced registration settings"));
	regSettingsAct->setShortcut(tr("Ctrl+O"));
	connect(regSettingsAct, SIGNAL(triggered()), this, SLOT(showSettingsDialog()));
	
	//view menu actions
	zoom1Act = new QAction(tr("&100%"), this);
	zoom1Act->setShortcut(tr("Ctrl+1"));
	zoom1Act->setStatusTip(tr("Zoom the images 100% size"));
	zoom1Act->setCheckable(true);
	connect(zoom1Act, SIGNAL(triggered()), this, SLOT(setZoom1()));
	
	zoom2Act = new QAction(tr("&200%"), this);
	zoom2Act->setShortcut(tr("Ctrl+2"));
	zoom2Act->setStatusTip(tr("Zoom the images to 200% size"));
	zoom2Act->setCheckable(true);
	//zoom2Act->setChecked(true);
	connect(zoom2Act, SIGNAL(triggered()), this, SLOT(setZoom2()));
	
	zoom3Act = new QAction(tr("&300%"), this);
	zoom3Act->setShortcut(tr("Ctrl+3"));
	zoom3Act->setStatusTip(tr("Zoom the images to 300% size"));
	zoom3Act->setCheckable(true);
	connect(zoom3Act, SIGNAL(triggered()), this, SLOT(setZoom3()));
	
	zoom4Act = new QAction(tr("&400%"), this);
	zoom4Act->setShortcut(tr("Ctrl+4"));
	zoom4Act->setStatusTip(tr("Zoom the images to 400% size"));
	zoom4Act->setCheckable(true);
	connect(zoom4Act, SIGNAL(triggered()), this, SLOT(setZoom4()));
	
	zoom6Act = new QAction(tr("&600%"), this);
	zoom6Act->setShortcut(tr("Ctrl+6"));
	zoom6Act->setStatusTip(tr("Zoom the images to 600% size"));
	zoom6Act->setCheckable(true);
	//zoom6Act->setChecked(true); //New default value
	connect(zoom6Act, SIGNAL(triggered()), this, SLOT(setZoom6()));

	interp1Act = new QAction(tr("Fast"), this);
	interp1Act->setStatusTip(tr("Perform quick interpolation, no smoothing"));
	interp1Act->setCheckable(true);
	connect(interp1Act, SIGNAL(triggered()), this, SLOT(setInterp1()));

	interp2Act = new QAction(tr("Bilinear"), this);
	interp2Act->setStatusTip(tr("Perform bilinear filtering"));
	interp2Act->setCheckable(true);
	interp2Act->setChecked(true);
	connect(interp2Act, SIGNAL(triggered()), this, SLOT(setInterp2()));

	checkeredAct = new QAction(tr("Chec&kered Composite Image"), this);
	checkeredAct->setShortcut(tr("Ctrl+K"));
	checkeredAct->setStatusTip(tr("Show a checkerboard composite of the moving and fixed images"));
	checkeredAct->setCheckable(true);
	checkeredAct->setChecked(false);
	connect(checkeredAct, SIGNAL(triggered()), this, SLOT(showCheckered()));
	
	showVoiRegionAct = new QAction(tr("S&how VOI Region"), this);
	showVoiRegionAct->setShortcut(tr("Ctrl+H"));
	showVoiRegionAct->setStatusTip(tr("Show the VOI region in the registration display"));
	showVoiRegionAct->setCheckable(true);
	showVoiRegionAct->setChecked(false);
	connect(showVoiRegionAct, SIGNAL(triggered()), this, SLOT(showVoiRegion()));
	
	checkerboardOptionsAct = new QAction(tr("Checkerboard Vie&w Options"), this);
	checkerboardOptionsAct->setShortcut(tr("Ctrl+W"));
	checkerboardOptionsAct->setStatusTip(tr("Show The checkerboard view options dialog"));
	connect(checkerboardOptionsAct, SIGNAL(triggered()), this, SLOT(showCheckerboardOptionsDialog()));
	
	viewOptionsAct = new QAction(tr("Advanced Options"), this);
	viewOptionsAct->setStatusTip(tr("Set various advanced viewing options"));
	viewOptionsAct->setShortcut(tr("Ctrl+D"));
	connect(viewOptionsAct, SIGNAL(triggered()), this, SLOT(showViewOptionsDialog()));							 
	
	//help menu actions
	aboutAct = new QAction(tr("&About"), this);
	aboutAct->setShortcut(tr("Ctrl+A"));
	aboutAct->setStatusTip(tr("About PetSpectFusion"));
	connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));
	
	connect(xTransBox, SIGNAL(valueChanged(double)), this, SLOT(setTranslationParams(double)));
	connect(yTransBox, SIGNAL(valueChanged(double)), this, SLOT(setTranslationParams(double)));
	connect(zTransBox, SIGNAL(valueChanged(double)), this, SLOT(setTranslationParams(double)));
	connect(xVerRotBox, SIGNAL(valueChanged(double)), this, SLOT(setVersorParams(double)));
	connect(yVerRotBox, SIGNAL(valueChanged(double)), this, SLOT(setVersorParams(double)));
	connect(zVerRotBox, SIGNAL(valueChanged(double)), this, SLOT(setVersorParams(double)));
	connect(xEulRotBox, SIGNAL(valueChanged(double)), this, SLOT(setEulerParams(double)));
	connect(yEulRotBox, SIGNAL(valueChanged(double)), this, SLOT(setEulerParams(double)));
	connect(zEulRotBox, SIGNAL(valueChanged(double)), this, SLOT(setEulerParams(double)));
	connect(registerButton, SIGNAL(clicked()), this, SLOT(doReg()));
	connect(evaluateButton, SIGNAL(clicked()), this, SLOT(evalCurrent()));
}

void QPetSpectFusionViewer::createComponents()
{
	setCentralWidget(new QWidget());
	QVBoxLayout* mainLayout = new QVBoxLayout(centralWidget());
	
	regView = new QRegistrationView(this);
	mainLayout->addWidget(regView);

	QGridLayout *paramsGrid = new QGridLayout;
	QLabel *translLabel = new QLabel(tr("Translation Parameters"));
	translLabel->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	paramsGrid->addWidget(translLabel, 0, 1);
	QLabel *rotLabel = new QLabel(tr("Rotation Versor"));
	rotLabel->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	paramsGrid->addWidget(rotLabel, 0, 2);
	
	QLabel* xLabel = new QLabel(tr("X"));
	xLabel->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	paramsGrid->addWidget(xLabel, 1, 0);
	QLabel* yLabel = new QLabel(tr("Y"));
	yLabel->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	paramsGrid->addWidget(yLabel, 2, 0);
	QLabel* zLabel = new QLabel(tr("Z"));
	zLabel->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	paramsGrid->addWidget(zLabel, 3, 0);
	
	xTransBox = new QDoubleSpinBox();
	xTransBox->setSingleStep(2.0);
	xTransBox->setDecimals(3);
	xTransBox->setRange(-900.0, 900.0);
	xTransBox->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	paramsGrid->addWidget(xTransBox, 1, 1, Qt::AlignHCenter);

	yTransBox = new QDoubleSpinBox();
	yTransBox->setSingleStep(2.0);
	yTransBox->setDecimals(3);
	yTransBox->setRange(-900.0, 900.0);
	yTransBox->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	paramsGrid->addWidget(yTransBox, 2, 1, Qt::AlignHCenter);
	
	zTransBox = new QDoubleSpinBox();
	zTransBox->setSingleStep(2.0);
	zTransBox->setDecimals(3);
	zTransBox->setRange(-900.0, 900.0);
	zTransBox->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	paramsGrid->addWidget(zTransBox, 3, 1, Qt::AlignHCenter);
	
	xVerRotBox = new QDoubleSpinBox();
	xVerRotBox->setSingleStep(0.005);
	xVerRotBox->setDecimals(7);
	xVerRotBox->setRange(-0.9999, 0.9999);
	xVerRotBox->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	paramsGrid->addWidget(xVerRotBox, 1, 2, Qt::AlignHCenter);

	yVerRotBox = new QDoubleSpinBox();
	yVerRotBox->setSingleStep(0.005);
	yVerRotBox->setDecimals(7);
	yVerRotBox->setRange(-0.9999, 0.9999);
	yVerRotBox->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	paramsGrid->addWidget(yVerRotBox, 2, 2, Qt::AlignHCenter);
	
	zVerRotBox = new QDoubleSpinBox();
	zVerRotBox->setSingleStep(0.005);
	zVerRotBox->setDecimals(7);
	zVerRotBox->setRange(-0.9999, 0.9999);
	zVerRotBox->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	paramsGrid->addWidget(zVerRotBox, 3, 2, Qt::AlignHCenter);
	
	QLabel* eulerLabel = new QLabel(tr("Rotation Euler Angles"));
	eulerLabel->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	paramsGrid->addWidget(eulerLabel, 0, 3);
	
	xEulRotBox = new QDoubleSpinBox();
	xEulRotBox->setSingleStep(0.5);
	xEulRotBox->setRange(-180.0, 180.0);
	xEulRotBox->setDecimals(2);
	xEulRotBox->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	paramsGrid->addWidget(xEulRotBox, 1, 3, Qt::AlignHCenter);
	
	yEulRotBox = new QDoubleSpinBox();
	yEulRotBox->setSingleStep(0.5);
	yEulRotBox->setRange(-90.0, 90.0);
	yEulRotBox->setDecimals(2);
	yEulRotBox->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	paramsGrid->addWidget(yEulRotBox, 2, 3, Qt::AlignHCenter);
	
	zEulRotBox = new QDoubleSpinBox;
	zEulRotBox->setSingleStep(0.5);
	zEulRotBox->setRange(-180.0, 180.0);
	zEulRotBox->setDecimals(2);
	zEulRotBox->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	paramsGrid->addWidget(zEulRotBox, 3, 3, Qt::AlignHCenter);
	
	QGroupBox* transformBox = new QGroupBox(QString("Transform Settings"));
	transformBox->setLayout(paramsGrid);
	
	QGridLayout *progressGrid = new QGridLayout;
	QLabel *metricLbl = new QLabel(tr("Metric Value: "));
	progressGrid->addWidget(metricLbl, 0, 0);
	metricLabel = new QLabel(tr("0.0000"));
	progressGrid->addWidget(metricLabel, 0, 1);
	QLabel *iterLbl = new QLabel(tr("Iteration: "));
	progressGrid->addWidget(iterLbl, 0, 2);
	iterationLabel = new QLabel(tr("0"));
	progressGrid->addWidget(iterationLabel, 0, 3);
	
	QGroupBox* progressBox = new QGroupBox(tr("Progress"));
	progressBox->setLayout(progressGrid);
	
	QBoxLayout *buttonLayout = new QHBoxLayout;
	buttonLayout->setAlignment(Qt::AlignHCenter);
	evaluateButton = new QPushButton(tr("Evaluate Current Transform"));
	evaluateButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	buttonLayout->addWidget(evaluateButton, 4, Qt::AlignHCenter);
	QLabel* logoLabel = new QLabel;
	QPixmap logoPixmap(QString(":/img/logo.png"));
	logoLabel->setPixmap(logoPixmap);
	buttonLayout->addWidget(logoLabel, 5, Qt::AlignHCenter); 
	registerButton = new QPushButton(tr("Perform registration"));
	registerButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	buttonLayout->addWidget(registerButton, 4, Qt::AlignHCenter);
	
	QVBoxLayout *stackedLayout = new QVBoxLayout;
	stackedLayout->addWidget(transformBox);
	stackedLayout->addWidget(progressBox);
	stackedLayout->addLayout(buttonLayout);
	
	mainLayout->addLayout(stackedLayout);
}

void QPetSpectFusionViewer::createMenus()
{
	fileMenu = menuBar()->addMenu(tr("File"));
	fileMenu->addAction(openFixedImageAct);
	fileMenu->addAction(openMovingImageAct);
	
	saveMenu = fileMenu->addMenu("Save ...");
	saveMenu->addAction(saveAllAct);
	saveMenu->addAction(saveRegisteredAct);
	saveMenu->addAction(saveCheckDiffAct);
	saveMenu->addAction(saveAbsDiffAct);
	saveMenu->addAction(saveTransformParamsAct);
	
	fileMenu->addAction(exitAct);
	
	regMenu = menuBar()->addMenu(tr("Registration"));
	regMenu->addAction(doRegAct);
	regMenu->addAction(multiResAct);
	
	QMenu* regModeMenu = regMenu->addMenu(tr("Registration Mode"));
	regModeMenu->addAction(fullRegistrationAct);
	regModeMenu->addAction(voiRegistrationAct);
	regModeMenu->addAction(voiAndFullRegistrationAct);
	regMenu->addAction(regSettingsAct);

	viewMenu = menuBar()->addMenu(tr("View"));
	zoomMenu = viewMenu->addMenu(tr("Zoom"));
	zoomMenu->addAction(zoom1Act);
	zoomMenu->addAction(zoom2Act);
	zoomMenu->addAction(zoom3Act);
	zoomMenu->addAction(zoom4Act);
	zoomMenu->addAction(zoom6Act);

	interpolationMenu = viewMenu->addMenu(tr("Interpolation"));
	interpolationMenu->addAction(interp1Act);
	interpolationMenu->addAction(interp2Act);
	 
	viewMenu->addAction(checkeredAct);
	viewMenu->addAction(checkerboardOptionsAct);
	viewMenu->addAction(showVoiRegionAct);
	viewMenu->addAction(viewOptionsAct);
	
	helpMenu = menuBar()->addMenu(tr("Help"));
	helpMenu->addAction(aboutAct);
}

void QPetSpectFusionViewer::openFixedImage()
{
	statusBar()->showMessage(tr("Loading Fixed Image ..."));
	QString fileName = QFileDialog::getOpenFileName(this, tr("Open Fixed Image"),"",tr("Image file (*.mhd *.img);;Other files (*)"));
	QApplication::instance()->processEvents();
	if(fileName.isEmpty()) {
		statusBar()->showMessage(tr("No Image Loaded."));
		return;
	}
	
	openFixedFileName(fileName.toStdString());
}

void QPetSpectFusionViewer::openMovingImage()
{

	statusBar()->showMessage(tr("Loading Moving Image ..."));
	QString fileName = QFileDialog::getOpenFileName(this, tr("Open Moving Image"),"",tr("Image file (*.mhd *.img);; Other files (*)"));
	QApplication::instance()->processEvents();
	if(fileName.isEmpty()) {
		statusBar()->showMessage(tr("No Image Loaded."));
		return;
	}
	
	openMovingFileName(fileName.toStdString());
}

void QPetSpectFusionViewer::saveRegisteredImg()
{
	if(movingImage.IsNull() || fixedImage.IsNull())
		return;
	
	statusBar()->showMessage(tr("Saving registered Image..."));
	
	QString fileName = QFileDialog::getSaveFileName(this, tr("Save Registered Image"),
                            tr(outputFileName.c_str()),
                            tr("Image file (*.mhd *.img);;Other file (*)"));
	
	
	if(fileName.isEmpty()) {
		statusBar()->showMessage(tr("No Image Saved"));
		return;
	}
	
	outputFileName = fileName.toStdString();
	
	writeRegisteredImage(outputFileName.c_str(), fixedImage, movingImage, transformParams);
	
	statusBar()->showMessage(tr("Registered Image saved"));

}

void QPetSpectFusionViewer::saveCheckDiffImg()
{	
	if(movingImage.IsNull() || fixedImage.IsNull())
		return;
	
	statusBar()->showMessage(tr("Saving Checkerboard Composite Image..."));
	
	QString fileName = QFileDialog::getSaveFileName(this, tr("Save Checkerboard Image"),
                            "",
                            tr("Image file (*.mhd)"));
	
	
	if(fileName.isEmpty()) {
		statusBar()->showMessage(tr("No Image Saved"));
		return;
	}
	
	typedef itk::ResampleImageFilter<InternalImageType, InternalImageType> ResampleFilterType;
	ResampleFilterType::Pointer resampler = ResampleFilterType::New();

	TransformType::Pointer transform = TransformType::New();
	initializeTransform(transform, fixedImage, movingImage);
	transform->SetParameters(transformParams);

	resampler->SetTransform(transform);
	resampler->SetInput(movingImage);
	resampler->SetSize(fixedImage->GetLargestPossibleRegion().GetSize());
	resampler->SetOutputOrigin( fixedImage->GetOrigin());
	resampler->SetOutputSpacing( fixedImage->GetSpacing());
	resampler->SetDefaultPixelValue(0);
	resampler->Update();
	

	writeOutputImageMhd(fileName.toStdString().c_str(), getCheckerBoardImage(fixedImage, resampler->GetOutput()));
	statusBar()->showMessage(tr("Checkerboard Composite Image saved"));

}

void QPetSpectFusionViewer::saveAbsDiffImg()
{
	if(movingImage.IsNull() || fixedImage.IsNull())
		return;
	
	statusBar()->showMessage(tr("Saving Absolute Difference Image..."));
	
	QString fileName = QFileDialog::getSaveFileName(this, tr("Save Absolute Difference Image"),
                            "",
                            tr("Image file (*.mhd)"));
	
	
	if(fileName.isEmpty()) {
		statusBar()->showMessage(tr("No Image Saved"));
		return;
	}
	
	typedef itk::ResampleImageFilter<InternalImageType, InternalImageType> ResampleFilterType;
	ResampleFilterType::Pointer resampler = ResampleFilterType::New();

	TransformType::Pointer transform = TransformType::New();
	initializeTransform(transform, fixedImage, movingImage);
	transform->SetParameters(transformParams);

	resampler->SetTransform(transform);
	resampler->SetInput(movingImage);
	resampler->SetSize(fixedImage->GetLargestPossibleRegion().GetSize());
	resampler->SetOutputOrigin( fixedImage->GetOrigin());
	resampler->SetOutputSpacing( fixedImage->GetSpacing());
	resampler->SetDefaultPixelValue(0);
	resampler->Update();

	writeOutputImageMhd(fileName.toStdString().c_str(), getAbsoluteDifference(fixedImage, resampler->GetOutput()));
	statusBar()->showMessage(tr("Absolute Difference Image saved"));
}

void QPetSpectFusionViewer::saveTransformParams()
{
	if(movingImage.IsNull() || fixedImage.IsNull())
		return;
	
	statusBar()->showMessage(tr("Saving Parameters..."));
	
	QString fileName = QFileDialog::getSaveFileName(this, tr("Save Parameters file"),
                            "",
                            tr("Text file (*.txt)"));
	
	if(fileName.isEmpty()) {
		statusBar()->showMessage(tr("No Image Saved"));
		return;
	}
	
	std::ofstream logFile;
	time_t rawtime;
	time(&rawtime);
	
//use secure c runtime enhancements when on windows
//http://msdn.microsoft.com/de-de/library/8ef0s5kh.aspx
#ifdef WIN32 
#define SIZE 26	/* Added by Axel to remove deprecation */
	char buf[SIZE];
	errno_t err;
	err = ctime_s( buf, SIZE, &rawtime );
#endif
	
	logFile.open(fileName.toStdString().c_str());
	
#ifdef WIN32
	logFile << "Parameters file generated: " << buf << std::endl;  /* ctime(&rawtime) */ 
#else
	logFile << "Parameters file generated: " << ctime(&rawtime) << std::endl;
#endif
	
	logFile << "fixed file name: " << fixedFileName << std::endl;
	logFile << "moving file name: " << movingFileName << std::endl;
	logFile << "versor X      = " << xVerRotBox->value() << std::endl;
	logFile << "versor Y      = " << yVerRotBox->value() << std::endl;
	logFile << "versor Z      = " << zVerRotBox->value() << std::endl;
	logFile << "Translation X = " << xTransBox->value() << std::endl;
	logFile << "Translation Y = " << yTransBox->value() << std::endl;
	logFile << "Translation Z = " << zTransBox->value()  << std::endl;
	evalCurrent();
	logFile << "Metric value = " << metricLabel->text().toStdString() << std::endl;
	logFile.close();

	statusBar()->showMessage(tr("Parameters saved"));
}

void QPetSpectFusionViewer::saveAll()
{
	if(movingImage.IsNull() || fixedImage.IsNull())
		return;
	
	saveRegisteredImg();
	saveCheckDiffImg();
	saveAbsDiffImg();
	saveTransformParams();
	
}

void QPetSpectFusionViewer::doReg()
{
	if(!regIsRunning && fixedImage.IsNotNull() && movingImage.IsNotNull())
	{
		std::ostringstream ostring(std::ostringstream::out);

		ostring << "Registration started" << std::endl;
		ostring << "fixed image file: " << fixedFileName << std::endl;
		ostring << "moving image file: " << movingFileName << std::endl;
		ostring << "output image file: " << outputFileName << std::endl;
		logging::logMessage(ostring);
		statusBar()->showMessage(tr("Registration running ..."));
		enableSettings(false);

		InternalImageType::Pointer fixedImagePointer;
		
		//if we are only performing a VOI registration, then extract the region now
		if(voiRegistrationAct->isChecked()) {
			
			logging::logMessage("Performing VOI registration");
			
			if(!checkVOIRegionBounds())
			{
				statusBar()->showMessage(tr("Registration aborted ..."));
				logging::logMessage("Region bounds invalid, registration aborted");
				QMessageBox::critical(this, QString("Registration Error"), 
									  QString("Error performing VOI registration, region bounds invalid"));
				enableSettings(false);				
				return;
			}
			
			typedef itk::ExtractImageFilter<InternalImageType, InternalImageType> RegionFilterType;
			RegionFilterType::Pointer regionFilter = RegionFilterType::New();
			regionFilter->SetInput(fixedImage);
			regionFilter->SetExtractionRegion(voiRegion);
			
			InternalImageType::Pointer voiImage = regionFilter->GetOutput();
			voiImage->Update();
			fixedImagePointer = voiImage;
		}
		else //otherwise perform the registration using the full fixed iamge
		{
			fixedImagePointer = fixedImage;
		}

		if(voiAndFullRegistrationAct->isChecked())
			isVoiAndFullFirstPass = true;
		
		regThread = new QRegistrationThread(this, this, fixedImagePointer, movingImage, bins, 
											minStepSize, maxStepSize, sampleRate, maxIterations, 
											optimizerScales, transformParams);
		
		if(plotting)
				regThread->enablePlotting(plotFileName);

		
		if(multiResAct->isChecked())
		{
			regThread->setMultiRes(true, levels);
		}
		
		connect(regThread, SIGNAL(finished()), this, SLOT(registrationDone()));
		connect(regThread, SIGNAL(parametersUpdated(ParametersType,double,int)),
			this, SLOT(registrationUpdate(ParametersType,double,int)));
		regThread->start();

		regIsRunning = true;
		
		evaluateButton->setDisabled(true);
		registerButton->setText(tr("Stop Registration"));
	}
		//if there was a problem loading the images and we are in automatic mode, quit the app
	else if(!regIsRunning && automatic_mode)
	{
		logging::logMessage("Error with Automatic registration!");
		close();
	}
	else if(regIsRunning)
	{
		statusBar()->showMessage(tr("Stopping the registration ..."));
		logging::logMessage("Telling registration thread to quit...");
		
		regThread->stopRegistration();
		registerButton->setDisabled(true);
	
	}
}

void QPetSpectFusionViewer::evalCurrent()
{
	if(movingImage.IsNull() || fixedImage.IsNull())
		return;

	statusBar()->showMessage(tr("Calculating the metric ..."));
	
	typedef itk::MattesMutualInformationImageToImageMetric< InternalImageType, InternalImageType> MetricType;
	
	typedef itk:: LinearInterpolateImageFunction< 
                                    InternalImageType,
                                    double          >    InterpolatorType;
		
	MetricType::Pointer         metric        = MetricType::New();
	InterpolatorType::Pointer   interpolator  = InterpolatorType::New();
	TransformType::Pointer  transform = TransformType::New();

	//initialize the metric for Mattes MI
	metric->SetNumberOfHistogramBins(bins);
	InternalImageType::RegionType fixedImageRegion = fixedImage->GetBufferedRegion();
	initializeTransform(transform, fixedImage, movingImage);
	
	metric->SetFixedImage(fixedImage);
	metric->SetMovingImage(movingImage);
	metric->SetInterpolator(interpolator);
	metric->SetTransform(transform);
	metric->SetFixedImageRegion(fixedImageRegion);
	metric->UseAllPixelsOn();

	try 
	{ 
		metric->Initialize();
	} 
	catch( itk::ExceptionObject & err ) 
	{ 
		logging::logError("Error calculating the metric value!!!"); 
		std::cerr << err << std::endl; 
		
		QMessageBox::critical(this, QString("Error writing image"), 
				QString("There was an error writing the image data see the console / log for more details."));
		
		return;
	} 

	metricLabel->setNum(metric->GetValue(transformParams));
	statusBar()->showMessage(tr("Metric calculated"));
}

void QPetSpectFusionViewer::about()
{
	QMessageBox::about(this, tr("About PetSpectFusion"),
             tr("PetSpectFusion is an automatic and interactive image registration program optimized "
				"for use with the nuclear cardiac image modalities Positron Emission Tomography (PET) and "
				"Single Photon Emission Computed Tomography (SPECT)<br /><br />"
				"This program was developed as part of an joint research project between the university hospital:<br />"
				"Klinikum Rechts der Isar<br />"
				"Nuklearmedizinische Klinik und Poliklinik<br /><br />"
				"and the department of computer science:<br />"
				"Chair for Computer Aided Medical Procedures and Augmented Reality<br /><br />"
				"Technische Universitaet Muenchen<br /><br />"
				"Developers:<br />"
				"Lead: Brian Jensen <a href=\"mailto:jensen@in.tum.de\">jensen@in.tum.de</a><br />"
				"Axel Martinez Moeller <a href=\"mailto:a.martinez-moller@lrz.tu-muenchen.de\">a.martinez-moller@lrz.tu-muenchen.de</a><br />"
				"Martina Marinelli <a href=\"marti.marinelli@gmail.com\">marti.marinelli@gmail.com</a>/<br />"
				"Dr. Stephan Nekolla <a href=\"mailto:s.nekolla@lrz.tu-muenchen.de\">s.nekolla@lrz.tu-muenchen.de</a><br /><br />"
				"PetSpectFusion version 1.31 (rev 74) 12/2009"
				));
}

void QPetSpectFusionViewer::setTranslationParams(double val)
{
	transformParams.SetElement(3, xTransBox->value());
	transformParams.SetElement(4, yTransBox->value());
	transformParams.SetElement(5, zTransBox->value());

	regView->setTransformParams(transformParams);
}

void QPetSpectFusionViewer::setVersorParams(double val)
{
	//prevent conversion oscillations between versors and Euler angles
	//only update angles if they weren't already being updated
	if(updateAngles)
	{
		//prevent any further any updates
		updateAngles = false;
		double x = xVerRotBox->value();
		double y = yVerRotBox->value();
		double z = zVerRotBox->value();
		if ( 1.0 - (x*x + y*y + z*z) <= 1e-10) 
		{
			logging::logMessage("Versor params too close to 1, unable to correctly normalize");
			xVerRotBox->setValue(transformParams[0]);
			yVerRotBox->setValue(transformParams[1]);
			zVerRotBox->setValue(transformParams[2]);
		}
		else 
		{
			transformParams[0] = x;
			transformParams[1] = y;
			transformParams[2] = z;
		
			RotationType versor(3);
			versor[0] = x;
			versor[1] = y;
			versor[2] = z;
		
			RotationType eulerAngles = convertVersorToEuler(versor);
				
			xEulRotBox->setValue(eulerAngles[0]);
			yEulRotBox->setValue(eulerAngles[1]);
			zEulRotBox->setValue(eulerAngles[2]);

			regView->setTransformParams(transformParams);
		}
		
		updateAngles = true;
	}
}

void QPetSpectFusionViewer::setEulerParams(double val)
{	
	//prevent conversion oscillations between versors and Euler angles
	//only update angles if they weren't already being updated
	if(updateAngles)
	{
		//prevent further automatic updates
		updateAngles = false;
		RotationType eulerAngles(3);
		eulerAngles[0] = xEulRotBox->value();
		eulerAngles[1] = yEulRotBox->value();
		eulerAngles[2] = zEulRotBox->value();
	
		RotationType versor = convertEulerToVersor(eulerAngles);
		xVerRotBox->setValue(versor[0]);
		yVerRotBox->setValue(versor[1]);
		zVerRotBox->setValue(versor[2]);

		transformParams[0] = versor[0];
		transformParams[1] = versor[1];
		transformParams[2] = versor[2];
		
		regView->setTransformParams(transformParams);

		updateAngles = true;
	}
}

/**
 *	Function is called by the registration observer to update the display with the current parameters
 *
 */
void QPetSpectFusionViewer::registrationUpdate(ParametersType curParams, double metricVal, int iteration)
{
	//prevent conversion oscillations between versors and Euler angles
	updateAngles = false;
	transformParams = curParams;
	
	xVerRotBox->setValue(curParams.GetElement(0));
	yVerRotBox->setValue(curParams.GetElement(1));
	zVerRotBox->setValue(curParams.GetElement(2));
	xTransBox->setValue(curParams.GetElement(3));
	yTransBox->setValue(curParams.GetElement(4));
	zTransBox->setValue(curParams.GetElement(5));
	
	RotationType versor(3);
	versor[0] = curParams[0];
	versor[1] = curParams[1];
	versor[2] = curParams[2];
	
	RotationType eulerAngles = convertVersorToEuler(versor);
	xEulRotBox->setValue(eulerAngles[0]);
	yEulRotBox->setValue(eulerAngles[1]);
	zEulRotBox->setValue(eulerAngles[2]);
	
	metricLabel->setNum(metricVal);
	iterationLabel->setNum(iteration);
	
	regView->setTransformParams(transformParams);
	
	updateAngles = true;
}

/**
 *	Function is used internally to initialize the spin boxes with the transform params passed from the command line
 *
 */
void QPetSpectFusionViewer::setParameters(ParametersType curParams)
{
	//prevent conversion oscillations between versors and Euler angles
	updateAngles = false;
	
	xVerRotBox->setValue(curParams.GetElement(0));
	yVerRotBox->setValue(curParams.GetElement(1));
	zVerRotBox->setValue(curParams.GetElement(2));
	xTransBox->setValue(curParams.GetElement(3));
	yTransBox->setValue(curParams.GetElement(4));
	zTransBox->setValue(curParams.GetElement(5));

	RotationType versorParams(3);
	versorParams[0] = curParams[0];
	versorParams[1] = curParams[1];
	versorParams[2] = curParams[2];
	
	RotationType eulerAngles = convertVersorToEuler(versorParams);
	
	xEulRotBox->setValue(eulerAngles[0]);
	yEulRotBox->setValue(eulerAngles[1]);
	zEulRotBox->setValue(eulerAngles[2]);
	
	//Martina: make sure this code works properly, to fix the error of no geometric initialization in the automatic mode without GUI
	transformParams = curParams;
	regView->setTransformParams(transformParams);
	
	updateAngles = true;
}

/**
 *	Signal is called when the registration thread terminates
 *
 */
void QPetSpectFusionViewer::registrationDone()
{	
	//flag to determine if the registration should be started a second time in VOI mode
	if(isVoiAndFullFirstPass)
	{
		isVoiAndFullFirstPass = false;
		
		logging::logMessage("Performing VOI registration");
		
		if(!checkVOIRegionBounds())
		{
			statusBar()->showMessage(tr("Registration aborted ..."));
			logging::logMessage("Region bounds invalid, registration aborted");
			QMessageBox::critical(this, QString("Registration Error"), 
								  QString("Error performing VOI registration, region bounds invalid"));
			enableSettings(false);				
			return;
		}
		
		//clear out the angle
		xVerRotBox->setValue(0);
		yVerRotBox->setValue(0);
		zVerRotBox->setValue(0);
		
		typedef itk::ExtractImageFilter<InternalImageType, InternalImageType> RegionFilterType;
		RegionFilterType::Pointer regionFilter = RegionFilterType::New();
		regionFilter->SetInput(fixedImage);
		regionFilter->SetExtractionRegion(voiRegion);
		
		InternalImageType::Pointer voiImage = regionFilter->GetOutput();
		voiImage->Update();
	
		delete regThread;
		
		regThread = new QRegistrationThread(this, this, voiImage, movingImage, bins, 
										minStepSize, maxStepSize, sampleRate, maxIterations, 
										optimizerScales, transformParams);
	
		if(plotting)
			regThread->enablePlotting(plotFileName);
	
		if(multiResAct->isChecked())
			regThread->setMultiRes(true, levels);
		
		connect(regThread, SIGNAL(finished()), this, SLOT(registrationDone()));
		connect(regThread, SIGNAL(parametersUpdated(ParametersType,double,int)),
				this, SLOT(registrationUpdate(ParametersType,double,int)));
		regThread->start();
	}
	else 
	{
		logging::logMessage("Registration Completed!");
	
		enableSettings(true);
		
		registerButton->setText(tr("Perform registration"));
		
		evaluateButton->setDisabled(false);
		registerButton->setDisabled(false);
		regIsRunning = false;
	
		statusBar()->showMessage(tr("Registration completed"));
	}
}

/**
 *	The following functions set the zoomfactor used in displaying the slice pixmaps
 *
 */
void QPetSpectFusionViewer::setZoom1()
{
	regView->setZoomFactor(1);
	zoom1Act->setChecked(true);
	zoom2Act->setChecked(false);
	zoom3Act->setChecked(false);
	zoom4Act->setChecked(false);
	zoom6Act->setChecked(false);
}

void QPetSpectFusionViewer::setZoom2()
{
	regView->setZoomFactor(2);
	zoom1Act->setChecked(false);
	zoom2Act->setChecked(true);
	zoom3Act->setChecked(false);
	zoom4Act->setChecked(false);
	zoom6Act->setChecked(false);

}

void QPetSpectFusionViewer::setZoom3()
{
	regView->setZoomFactor(3);
	zoom1Act->setChecked(false);
	zoom2Act->setChecked(false);
	zoom3Act->setChecked(true);
	zoom4Act->setChecked(false);
	zoom6Act->setChecked(false);

}

void QPetSpectFusionViewer::setZoom4()
{
	regView->setZoomFactor(4);
	zoom1Act->setChecked(false);
	zoom2Act->setChecked(false);
	zoom3Act->setChecked(false);
	zoom4Act->setChecked(true);
	zoom6Act->setChecked(false);

}

void QPetSpectFusionViewer::setZoom6()
{
	regView->setZoomFactor(6);
	zoom1Act->setChecked(false);
	zoom2Act->setChecked(false);
	zoom3Act->setChecked(false);
	zoom4Act->setChecked(false);
	zoom6Act->setChecked(true);

}

/**
 *	The following functions set the interpolation mode used in displaying the slice pixmaps
 *
 */
void QPetSpectFusionViewer::setInterp1()
{
	regView->setInterpolationMode(1);
	interp1Act->setChecked(true);
	interp2Act->setChecked(false);

}

void QPetSpectFusionViewer::setInterp2()
{
	regView->setInterpolationMode(2);
	interp1Act->setChecked(false);
	interp2Act->setChecked(true);

}

void QPetSpectFusionViewer::showCheckered()
{
	regView->showCheckeredImage(checkeredAct->isChecked());
}

/**
 *	funcion greys out some options while the registration is running
 */
void QPetSpectFusionViewer::enableSettings(bool enable)
{
	xVerRotBox->setEnabled(enable);
	yVerRotBox->setEnabled(enable);
	zVerRotBox->setEnabled(enable);
	xEulRotBox->setEnabled(enable);
	yEulRotBox->setEnabled(enable);
	zEulRotBox->setEnabled(enable);
	xTransBox->setEnabled(enable);
	yTransBox->setEnabled(enable);
	zTransBox->setEnabled(enable);
		
	regSettingsAct->setEnabled(enable);
	openFixedImageAct->setEnabled(enable);
	openMovingImageAct->setEnabled(enable);
	saveRegisteredAct->setEnabled(enable);
	saveAbsDiffAct->setEnabled(enable);
	saveCheckDiffAct->setEnabled(enable);
	saveTransformParamsAct->setEnabled(enable);
	saveAllAct->setEnabled(enable);
	multiResAct->setEnabled(enable);
	
	fullRegistrationAct->setEnabled(enable);
	voiRegistrationAct->setEnabled(enable);
	voiAndFullRegistrationAct->setEnabled(enable);
	
}

/**
 *	Function sets a one shot timer that causes the registration to begin once the app is fully loaded
 *
 */
void QPetSpectFusionViewer::doAutomaticReg()
{
	automatic_mode = true;
	QTimer::singleShot(200, this, SLOT(doReg()));
}

void QPetSpectFusionViewer::setMultiRes(int levs)
{
	multiResAct->setChecked(true);
	levels = levs;

}

void QPetSpectFusionViewer::setPlotFile(std::string filename)
{
	plotFileName = filename;
	//plotsBox->setCheckState(Qt::Checked);
}

void QPetSpectFusionViewer::showSettingsDialog()
{
	if(!regSettingsDialogOpen)
	{
		logging::logMessage("Registration options dialog opened");
		settingsDialog = new RegSettingsDialog(this);
	
		//set the values for the dialog
		settingsDialog->binsBox->setValue(bins);
		settingsDialog->minSizeBox->setValue(minStepSize);
		settingsDialog->maxSizeBox->setValue(maxStepSize);
		settingsDialog->samplesBox->setValue(sampleRate * 100.0);
		settingsDialog->logNameBox->setText(QString(""));
		settingsDialog->loggingBox->setChecked(logging::fileLoggingEnabled());
		settingsDialog->plotsNameBox->setText(QString(plotFileName.c_str()));
		settingsDialog->plotsBox->setChecked(plotting);
	
		settingsDialog->xRotBox->setValue(optimizerScales[0]);
		settingsDialog->yRotBox->setValue(optimizerScales[1]);
		settingsDialog->zRotBox->setValue(optimizerScales[2]);
		settingsDialog->xTransBox->setValue(optimizerScales[3]);
		settingsDialog->yTransBox->setValue(optimizerScales[4]);
		settingsDialog->zTransBox->setValue(optimizerScales[5]);
	
		connect(settingsDialog, SIGNAL(destroyed()), this, SLOT(regSettingsDialogClosed()));
		
		regSettingsDialogOpen = true;
	}
	
	settingsDialog->show();
}

void QPetSpectFusionViewer::regSettingsDialogClosed()
{
	logging::logMessage("Registrtion options dialog closed");
	regSettingsDialogOpen = false;
	settingsDialog = NULL;	
}

std::string& QPetSpectFusionViewer::getFixedFileName()
{
	return fixedFileName;
}

std::string& QPetSpectFusionViewer::getMovingFileName()
{
	return movingFileName;
}

std::string& QPetSpectFusionViewer::getOutputFileName()
{
	return outputFileName;
}

bool QPetSpectFusionViewer::isRegistrationRunning()
{		
	return regIsRunning;
}

QRegistrationView* QPetSpectFusionViewer::getRegistrationView()
{
	return regView;
}

void QPetSpectFusionViewer::regViewDialogClosed()
{
	logging::logMessage("View Options closed");
	regViewDialogOpen = false;
	regViewDialog = NULL;
}

void QPetSpectFusionViewer::showViewOptionsDialog()
{
	if(regViewDialogOpen)
		regViewDialog->show();
	else {
		regViewDialog = new RegistrationViewDialog(this, regView);
		regViewDialog->show();
		regViewDialogOpen = true;
	}
}

ParametersType QPetSpectFusionViewer::getParameters()
{
	return transformParams;
}

double QPetSpectFusionViewer::getMetricVal()
{
	return metricLabel->text().toDouble();
}

int QPetSpectFusionViewer::getIteration()
{	
	return iterationLabel->text().toInt();
}

void QPetSpectFusionViewer::openFixedFileName(std::string fileName)
{	
	fixedImage = loadMhdImage(fileName.c_str());
	fixedFileName = fileName;
	if(fixedImage.IsNotNull())
	{
		regView->setFixedImage(fixedImage);
		statusBar()->showMessage(tr("Fixed Image loaded"));
	}
	else
		statusBar()->showMessage(tr("Fixed image failed to load!"));
}

void QPetSpectFusionViewer::openMovingFileName(std::string fileName)
{	
	movingImage = loadMhdImage(fileName.c_str());
	movingFileName = fileName;
	if(movingImage.IsNotNull())
	{
		regView->setMovingImage(movingImage);
		statusBar()->showMessage(tr("Moving Image loaded"));
	}
	else
		statusBar()->showMessage(tr("Moving image failed to load!"));
}

void QPetSpectFusionViewer::showCheckerboardOptionsDialog()
{
	if(checkerboardViewDialogOpen)
		checkerboardViewDialog->show();
	else {
		checkerboardViewDialog = new CheckerboardViewDialog(this, regView);
		checkerboardViewDialog->show();
		checkerboardViewDialogOpen = true;
	}
}

void QPetSpectFusionViewer::applyNewSettings()
{
	if(settingsDialog != NULL)
	{
		bins = settingsDialog->binsBox->value();
		minStepSize = settingsDialog->minSizeBox->value();
		maxStepSize = settingsDialog->maxSizeBox->value();
		sampleRate = settingsDialog->samplesBox->value() / 100.0;
		plotting = settingsDialog->plotsBox->isChecked();
		plotFileName = settingsDialog->plotsNameBox->text().toStdString();
		
		optimizerScales[0] = settingsDialog->xRotBox->value();
		optimizerScales[1] = settingsDialog->yRotBox->value();
		optimizerScales[2] = settingsDialog->zRotBox->value();
		optimizerScales[3] = settingsDialog->xTransBox->value();
		optimizerScales[4] = settingsDialog->yTransBox->value();
		optimizerScales[5] = settingsDialog->zTransBox->value();
		
		voiRegion.SetIndex(0, settingsDialog->indexX->value());
		voiRegion.SetIndex(1, settingsDialog->indexY->value());
		voiRegion.SetIndex(2, settingsDialog->indexZ->value());
		voiRegion.SetSize(0, settingsDialog->rangeX->value());
		voiRegion.SetSize(1, settingsDialog->rangeY->value());
		voiRegion.SetSize(2, settingsDialog->rangeZ->value());
		
		regView->setVoiRegion(voiRegion);
		regView->updateView();
		
		std::string logFileName;
		
		if(logging::fileLoggingEnabled())
		{
			if(settingsDialog->loggingBox->isChecked())
			{
				if(settingsDialog->logNameBox->text().toStdString() != logging::getLogFileName())
				{
					logFileName = settingsDialog->logNameBox->text().toStdString(); 
					logging::disableFileLogging();
					if(logFileName != "")
					{
						logging::enableFileLogging(logFileName);
						if(logging::fileLoggingEnabled())
						{
							logging::logMessage("File Logging Activated");
						}
						else
							std::cerr << "Error opening log file " << logFileName << " logging is disabled" << std::endl;
					}
				}
			}
			else
			{
				logging::disableFileLogging();
			}
		}
		//logging was off, now turned on
		else if(settingsDialog->loggingBox->isChecked() && settingsDialog->logNameBox->text().toStdString() != "")
		{
			logFileName = settingsDialog->logNameBox->text().toStdString();
			logging::enableFileLogging(logFileName);
			if(logging::fileLoggingEnabled())
			{
				logging::logMessage("File Logging Activated");
			}
			else
				std::cerr << "Error opening log file " << logFileName << " logging is disabled" << std::endl;
		}
	}
	
}

void QPetSpectFusionViewer::showVoiRegion()
{
	regView->showVoi(showVoiRegionAct->isChecked());
	regView->updateView();
}

void QPetSpectFusionViewer::setVoiMode()
{
	fullRegistrationAct->setChecked(false);
	voiRegistrationAct->setChecked(true);
	voiAndFullRegistrationAct->setChecked(false);	
}

void QPetSpectFusionViewer::setFullMode()
{
	fullRegistrationAct->setChecked(true);
	voiRegistrationAct->setChecked(false);
	voiAndFullRegistrationAct->setChecked(false);	
}

void QPetSpectFusionViewer::setVoiAndFullMode()
{
	fullRegistrationAct->setChecked(false);
	voiRegistrationAct->setChecked(false);
	voiAndFullRegistrationAct->setChecked(true);	
}
	
bool QPetSpectFusionViewer::checkVOIRegionBounds()
{
	//verify the bounds of the region, truncate if necessary
	RegionType fixedImageRegion = fixedImage->GetLargestPossibleRegion();
	if(voiRegion.GetIndex(0) < fixedImageRegion.GetIndex(0))
		voiRegion.SetIndex(0, fixedImageRegion.GetIndex(0));
	else if(voiRegion.GetIndex(0) >  fixedImageRegion.GetIndex(0) + (signed int) fixedImageRegion.GetSize(0))
		voiRegion.SetIndex(0, fixedImageRegion.GetIndex(0) + fixedImageRegion.GetSize(0));
	else if(voiRegion.GetIndex(0) + voiRegion.GetSize(0) >
			fixedImageRegion.GetIndex(0) + fixedImageRegion.GetSize(0))
		voiRegion.SetSize(0, voiRegion.GetIndex(0) - fixedImageRegion.GetIndex(0) + fixedImageRegion.GetSize(0));
	
	if(voiRegion.GetIndex(1) < fixedImageRegion.GetIndex(1))
		voiRegion.SetIndex(1, fixedImageRegion.GetIndex(1));
	else if( voiRegion.GetIndex(1) > fixedImageRegion.GetIndex(1) + (signed int) fixedImageRegion.GetSize(1))
		voiRegion.SetIndex(1, fixedImageRegion.GetIndex(1) + fixedImageRegion.GetSize(1));
	else if(voiRegion.GetIndex(1) + voiRegion.GetSize(1) >
			fixedImageRegion.GetIndex(1) + fixedImageRegion.GetSize(1))
		voiRegion.SetSize(1, voiRegion.GetIndex(1) - fixedImageRegion.GetIndex(1) + fixedImageRegion.GetSize(1));
	
	if(voiRegion.GetIndex(2) < fixedImageRegion.GetIndex(2))
		voiRegion.SetIndex(2, fixedImageRegion.GetIndex(2));
	else if(voiRegion.GetIndex(2) > fixedImageRegion.GetIndex(2) + (signed int) fixedImageRegion.GetSize(2))
		voiRegion.SetIndex(2, fixedImageRegion.GetIndex(2) + fixedImageRegion.GetSize(2));
	else if(voiRegion.GetIndex(2) + voiRegion.GetSize(2) >
			fixedImageRegion.GetIndex(2) + fixedImageRegion.GetSize(2))
		voiRegion.SetSize(2, voiRegion.GetIndex(2) - fixedImageRegion.GetIndex(2) + fixedImageRegion.GetSize(2));
	
	//now make sure region size is large enough
	//minimum size in each direction is 4 voxels
	if(voiRegion.GetSize(0) < 4 || voiRegion.GetSize(1) < 4 || voiRegion.GetSize(2) < 4)
		return false;
	
	return true;
}

RegionType QPetSpectFusionViewer::getVOIRegion()
{		
	return voiRegion;
}

RegSettingsDialog* QPetSpectFusionViewer::getRegSettingsDialog()
{
	return settingsDialog;
}

