//
//  $Id$
//  $Lastlog$
//
//  Registration.cxx
//  PetSpectFusion
//
//	Created 16/12/2007, Implements the registration functionality.
//  Copyright (c) 2007 - 2009 Brian Jensen. All rights reserved.
//

#define USE_MATH_DEFINES

#include "Log.h"
#include "typedefs.h"
#include "Registration.h"
#include "RegObserver.h"
#include "PetSpectFusionApp.h"


//Fix for windows environment
#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

#include <math.h>
#include <QtGui>
#include <sstream>

//Multhreading parameter, initialized to 0 which defaults to the number of processors on the system
int thread_count = 0;

using namespace logging;

RegistrationResultType doMattesRegistration(InternalImageType::Pointer fixedImage, InternalImageType::Pointer movingImage, 
		CommandIterationUpdate* observer, int bins, float sampleRate, float minStepSize, float maxStepSize, int maxIterations, 
		OptimizerScalesType optimizerScales, ParametersType tParams)
{	
	typedef itk::MattesMutualInformationImageToImageMetric< InternalImageType, InternalImageType> MetricType;
	
	typedef itk:: LinearInterpolateImageFunction< 
                                    InternalImageType,
                                    double          >    InterpolatorType;
	
	//set the number of threads here, but only if the user specified it explicitly
	if(thread_count)
		itk::MultiThreader::SetGlobalDefaultNumberOfThreads(thread_count);
	
	MetricType::Pointer         metric        = MetricType::New();
	OptimizerType::Pointer      optimizer     = OptimizerType::New();
	InterpolatorType::Pointer   interpolator  = InterpolatorType::New();
	RegistrationType::Pointer   registration  = RegistrationType::New();

	//initialize the transform
	TransformType::Pointer transform = TransformType::New();
	initializeTransform(transform, fixedImage, movingImage);
	
	if(tParams.Size() == 6)
	{
		transform->SetParameters(tParams);
	}
	
	std::ostringstream ostring(std::ostringstream::out);
	ostring << "Registration Parameters" << std::endl;
	ostring << "number of bins: " << bins << std::endl;
	ostring << "sample rate: " << sampleRate << std::endl;
	ostring << "min step size: " << minStepSize << std::endl;
	ostring << "max step size: " << maxStepSize << std::endl;
	ostring << "max iterations: " << maxIterations << std::endl;
	ostring << "optimizer scales: " << optimizerScales << std::endl;
	ostring << "transform params: " << transform->GetParameters() << std::endl << std::endl;
	logMessage(ostring);
	ostring.str("");
	
	registration->SetMetric(metric);
	registration->SetOptimizer(optimizer);
	registration->SetInterpolator(interpolator);
	registration->SetTransform( transform );
	
	registration->SetFixedImage(fixedImage);
	registration->SetMovingImage(movingImage);

	InternalImageType::RegionType fixedImageRegion = fixedImage->GetBufferedRegion();
	registration->SetFixedImageRegion(fixedImageRegion);

	//initialize the metric for Mattes MI
	metric->SetNumberOfHistogramBins(bins);
	const unsigned int numberOfPixels = fixedImageRegion.GetNumberOfPixels();
	const unsigned int numberOfSamples = static_cast< unsigned int >(sampleRate * numberOfPixels);
	metric->SetNumberOfSpatialSamples(numberOfSamples);

	//make sure we also use the same seed to avoid non determinism
	metric->ReinitializeSeed(12873465);
	
	registration->SetInitialTransformParameters( transform->GetParameters() );
	
		//initialize the optimizer step values
	optimizer->SetScales(optimizerScales);
	optimizer->SetMaximumStepLength(maxStepSize); 
	optimizer->SetMinimumStepLength(minStepSize);
	optimizer->MinimizeOn();
	optimizer->SetNumberOfIterations(maxIterations);

	optimizer->AddObserver( itk::IterationEvent(), observer );	
	RegistrationResultType result;
	
	//now do the registration!
	try 
    { 
		registration->StartRegistration(); 
    } 
	catch( itk::ExceptionObject & err ) 
    { 
		result.error = 10;
		logError("Error performing the registration!"); 
		std::cerr << err << std::endl; 
		
		if(PetSpectFusionApp::graphics_mode && PetSpectFusionApp::mainViewer)
			QMessageBox::critical(PetSpectFusionApp::mainViewer, QString("Error Performing Registration"), 
				QString("There was an error performing the registration see the console / log for more details"));
		
		return result;
    } 
	
	ostring << "Registration results:" << std::endl;
	ostring << "Last parameters: " << registration->GetLastTransformParameters() << std::endl;
	ostring << "Iterations: " << optimizer->GetCurrentIteration() << std::endl;
	ostring << "Metric Value: " << optimizer->GetValue() << std::endl;
	logMessage(ostring);
	
	result.error = 0;
	result.params = registration->GetLastTransformParameters();
	result.iteration = optimizer->GetCurrentIteration();
	result.metricVal = optimizer->GetValue();
	return result;
}

void initializeTransform(TransformType::Pointer transform, InternalImageType::Pointer fixedImage, InternalImageType::Pointer movingImage)
{	
	typedef itk::CenteredTransformInitializer< TransformType, 
                                             InternalImageType, 
                                             InternalImageType 
											 > TransformInitializerType;
	
	TransformInitializerType::Pointer initializer = 
                                          TransformInitializerType::New();

	initializer->SetTransform(transform);
	initializer->SetFixedImage(fixedImage);
	initializer->SetMovingImage(movingImage);
	initializer->GeometryOn();
	initializer->InitializeTransform();	
	
	//used for setting the initial transform
	typedef TransformType::VersorType  VersorType;
	typedef VersorType::VectorType     VectorType;

	VersorType     rotation;
	VectorType     axis;
  
	axis[0] = 0.0;
	axis[1] = 0.0;
	axis[2] = 1.0;

	const double angle = 0;

	rotation.Set(  axis, angle  );

	transform->SetRotation( rotation );
	
}

RegistrationResultType doMattesMultiRegistration(InternalImageType::Pointer fixedImage, InternalImageType::Pointer movingImage, 
		CommandIterationUpdate* observer, int bins, float sampleRate, float minStepSize, float maxStepSize, int maxIterations,
		OptimizerScalesType optimizerScales, int levels, ParametersType tParams)
{	
	typedef itk::MattesMutualInformationImageToImageMetric< InternalImageType, InternalImageType> MetricType;
	
	typedef itk:: LinearInterpolateImageFunction< 
                                    InternalImageType,
                                    double          >    InterpolatorType;
	//set the number of threads here, but only if the user specified it explicitly
	if(thread_count)
		itk::MultiThreader::SetGlobalDefaultNumberOfThreads(thread_count);
	
	MetricType::Pointer         metric        = MetricType::New();
	OptimizerType::Pointer      optimizer     = OptimizerType::New();
	InterpolatorType::Pointer   interpolator  = InterpolatorType::New();
	MultiResRegistrationType::Pointer registration = MultiResRegistrationType::New();
	
	InternalImagePyramidType::Pointer fixedImagePyramid = InternalImagePyramidType::New();
	InternalImagePyramidType::Pointer movingImagePyramid = InternalImagePyramidType::New();
	
	//initialize the transform
	TransformType::Pointer transform = TransformType::New();
	initializeTransform(transform, fixedImage, movingImage);
	
	if(tParams.Size() == 6)
	{
		transform->SetParameters(tParams);
	}
	
	std::ostringstream ostring(std::ostringstream::out);
	
	ostring << "Registration Parameters" << std::endl;
	ostring << "number of bins: " << bins << std::endl;
	ostring << "sample rate: " << sampleRate << std::endl;
	ostring << "min step size: " << minStepSize << std::endl;
	ostring << "max step size: " << maxStepSize << std::endl;
	ostring << "max iterations: " << maxIterations << std::endl;
	ostring << "optimizer scales: " << optimizerScales << std::endl;
	ostring << "transform params: " << transform->GetParameters() << std::endl;
	ostring << "multires levels: " << levels << std::endl << std::endl;
	logMessage(ostring);
	ostring.str("");
	
	registration->SetMetric(metric);
	registration->SetOptimizer(optimizer);
	registration->SetInterpolator(interpolator);
	registration->SetTransform( transform );
	
	registration->SetFixedImagePyramid(fixedImagePyramid);
	registration->SetMovingImagePyramid(movingImagePyramid);
	
	registration->SetFixedImage(fixedImage);
	registration->SetMovingImage(movingImage);


	InternalImageType::RegionType fixedImageRegion = fixedImage->GetBufferedRegion();
	registration->SetFixedImageRegion(fixedImageRegion);

	//initialize the metric for Mattes MI
	metric->SetNumberOfHistogramBins(bins);
	const unsigned int numberOfPixels = fixedImageRegion.GetNumberOfPixels();
	const unsigned int numberOfSamples = static_cast< unsigned int >(sampleRate * numberOfPixels);
	metric->SetNumberOfSpatialSamples(numberOfSamples);

	//make sure we also use the same seed to avoid non determinism
	metric->ReinitializeSeed(12873465);
	
	registration->SetInitialTransformParameters( transform->GetParameters() );
	
		//initialize the optimizer step values
	optimizer->SetScales(optimizerScales);
	optimizer->SetMaximumStepLength(maxStepSize); 
	optimizer->SetMinimumStepLength(minStepSize);
	optimizer->MinimizeOn();
	optimizer->SetNumberOfIterations(maxIterations);

	optimizer->AddObserver( itk::IterationEvent(), observer );
	
	typedef RegistrationInterfaceCommand<MultiResRegistrationType> CommandType;
	CommandType::Pointer command = CommandType::New();
	registration->AddObserver( itk::IterationEvent(), command );
	
	registration->SetNumberOfLevels( levels );
	fixedImagePyramid->SetStartingShrinkFactors(8);
	movingImagePyramid->SetStartingShrinkFactors(8);

	RegistrationResultType result;

	//now do the registration!
	try 
    { 
		registration->StartRegistration(); 
    } 
	catch( itk::ExceptionObject & err ) 
    { 
		logError("Error performing the registration!"); 
		std::cerr << err << std::endl; 
		result.error = 15;
		
		if(PetSpectFusionApp::graphics_mode && PetSpectFusionApp::mainViewer)
			QMessageBox::critical(PetSpectFusionApp::mainViewer, QString("Error Performing Registration"), 
				QString("There was an error performing the registration see the console / log for more details"));
				
		return result;
    } 
	
	ostring << "Registration results:" << std::endl;
	ostring << "Last parameters: " << registration->GetLastTransformParameters() << std::endl;
	ostring << "Iterations: " << optimizer->GetCurrentIteration() << std::endl;
	ostring << "Metric Value: " << optimizer->GetValue() << std::endl << std::endl;
	logMessage(ostring);
	
	result.error = 0;
	result.params = registration->GetLastTransformParameters();
	result.iteration = optimizer->GetCurrentIteration();
	result.metricVal = optimizer->GetValue();
	return result;
}

/**
 *		Conversion using XYZ Euler angle convention
 *
 */
RotationType convertVersorToEuler(RotationType versor)
{
	RotationType eulerParams(3);
	
	double x = versor[0];
	double y = versor[1];
	double z = versor[2];
	double w;
	
	double sqx = x*x;
	double sqy = y*y;
	double sqz = z*z;
	double norm = sqx + sqy + sqz;
	
	if(1.0 - norm <= 1e-10)
		w = 0;
	else 
		w = vcl_sqrt(1.0 - norm);
	
	/* FIX ME!
	double singTest = x * z + y * w;
	
	
	if (singTest > 0.499) // singularity at north pole
	{ 
		eulerParams[0] = vcl_atan2(x, w) / M_PI * 360.0;
		eulerParams[1] = 90.0;
		eulerParams[2] = 0.0;
	}
	else if (singTest < -0.499) // singularity at south pole
	{ 
		eulerParams[0] = - vcl_atan2(x, w) / M_PI * 360.0;
		eulerParams[1] = - 90.0;
		eulerParams[2] = 0.0;
	}
	else 
	 */
	{
		eulerParams[0] = vcl_atan2(2.0 * (w*x + y*z), 1.0 - 2.0 * (sqx + sqy)) / (2.0 * M_PI) * 360.0;
		eulerParams[1] = vcl_asin(2.0 * (w*y - z*x)) / (2.0 * M_PI) * 360.0;
		eulerParams[2] = vcl_atan2(2.0 * (w*z + x*y), 1.0 - 2.0 * (sqy + sqz)) / (2.0 * M_PI) * 360.0;
	}

	return eulerParams;
}

/**
 * Conversion using the XYZ Euler angle convention
 */
RotationType convertEulerToVersor(RotationType euler)
{
	RotationType versor(3);
	
	double c1 = vcl_cos(euler[0] * M_PI / 360.0);
    double s1 = vcl_sin(euler[0] * M_PI / 360.0);
    double c2 = vcl_cos(euler[1] * M_PI / 360.0);
    double s2 = vcl_sin(euler[1] * M_PI / 360.0);
    double c3 = vcl_cos(euler[2] * M_PI / 360.0);
    double s3 = vcl_sin(euler[2] * M_PI / 360.0);
	
  	versor[0] = s1*c2*c3 - c1*s2*s3;
	versor[1] = c1*s2*c3 + s1*c2*s3;
	versor[2] = c1*c2*s3 - s1*s2*c3;
	
	return versor;
}

