//
//  $Id$
//  $Lastlog$
//
//  RegistrationView.cxx
//  PetSpectFusion
//
//	Created 16/12/2007, Implements various functions for reading and writing images
//  Copyright (c) 2007 - 2009 Brian Jensen. All rights reserved.
//

#include <string.h>

#include "Log.h"
#include "PetSpectFusionViewer.h"
#include "Registration.h"
#include "RegistrationView.h"

const char *ColorTableNames[] = { "Hot Red", "Hot Blue", "Linear Red", "Black & White" };

/**
*	The constructor is responsible for setting up the layout of the widget and connecting all slots and signals
*/
QRegistrationView::QRegistrationView(QPetSpectFusionViewer* viewer)
{
	display = viewer;
	
	QGridLayout *gridView = new QGridLayout;
	QLabel *alphaTxtLabel = new QLabel(tr("Alpha"));
	gridView->addWidget(alphaTxtLabel, 0, 0);
	QLabel *transverLabel = new QLabel(tr("Transversal"));
	transverLabel->setAlignment(Qt::AlignHCenter);
	gridView->addWidget(transverLabel, 0, 1);
	QLabel *corLabel = new QLabel(tr("Coronal"));
	corLabel->setAlignment(Qt::AlignHCenter);
	gridView->addWidget(corLabel, 0, 2);
	QLabel *sagLabel = new QLabel(tr("Sagital"));
	sagLabel->setAlignment(Qt::AlignHCenter);
	gridView->addWidget(sagLabel, 0, 3);
	
	alphaSlider = new QSlider(Qt::Vertical);
	alphaSlider->setRange(0, 255);
	alphaSlider->setSliderPosition(128);
	alpha = 128;
	gridView->addWidget(alphaSlider, 1, 0, Qt::AlignHCenter);
	
	transViewLabel = new QLabel(this);
	gridView->addWidget(transViewLabel, 1, 1);

	corViewLabel = new QLabel(this);
	gridView->addWidget(corViewLabel, 1, 2);
	
	sagViewLabel = new QLabel(this);
	gridView->addWidget(sagViewLabel, 1, 3);
	
	alphaLabel = new QLabel(tr("50"));
	gridView->addWidget(alphaLabel, 2, 0, Qt::AlignHCenter | Qt::AlignVCenter);
	
	transSliceLabel = new QLabel(tr("0"));
	gridView->addWidget(transSliceLabel, 2, 1, Qt::AlignHCenter | Qt::AlignVCenter);
	
	corSliceLabel = new QLabel(tr("0"));
	gridView->addWidget(corSliceLabel, 2, 2, Qt::AlignHCenter | Qt::AlignVCenter);
	
	sagSliceLabel = new QLabel(tr("0"));
	gridView->addWidget(sagSliceLabel, 2, 3, Qt::AlignHCenter | Qt::AlignVCenter);
	
	transSliceSlider = new QSlider(Qt::Horizontal);
	gridView->addWidget(transSliceSlider, 3, 1);

	corSliceSlider = new QSlider(Qt::Horizontal);
	gridView->addWidget(corSliceSlider, 3, 2);
	
	sagSliceSlider = new QSlider(Qt::Horizontal);
	gridView->addWidget(sagSliceSlider, 3, 3);
	
	setLayout(gridView);

	//make sure to explicitly point the buffers to null, or else it will crash on windows
	fixed_trans_slice_buffer = NULL;
	fixed_cor_slice_buffer = NULL;
	fixed_sag_slice_buffer = NULL;
	moving_trans_slice_buffer = NULL;
	moving_cor_slice_buffer = NULL;
	moving_sag_slice_buffer = NULL;

	interpolationMode = 2;
	showCheckered = false;
	
	fixedImageColorTable = HotRed;
	movingImageColorTable = HotBlue;
	
	winMaxFixed = DEFAULT_WINDOW_MAX;
	winMinFixed = DEFAULT_WINDOW_MIN;
	winMaxMoving = DEFAULT_WINDOW_MAX;
	winMinMoving = DEFAULT_WINDOW_MIN;
	
	fixedImageMax = 0.0;
	fixedImageMin = 0.0;
	movingImageMax = 0.0;
	movingImageMin = 0.0;
	
	//checkerboard image order
	checkerFixedFirst = true;
	checkerPattern[0] = DEFAULT_CHECKER_SQUARES;
	checkerPattern[1] = DEFAULT_CHECKER_SQUARES;
	checkerPattern[2] = DEFAULT_CHECKER_SQUARES;
	
	transform = TransformType::New();
	
	useWindowRegion = false;
	showWinRegion = false;
	showRegistrationVoi = false;

	connect(transSliceSlider, SIGNAL(valueChanged(int)), this, SLOT(transSliceChanged(int)));
	connect(corSliceSlider, SIGNAL(valueChanged(int)), this, SLOT(corSliceChanged(int)));
	connect(sagSliceSlider, SIGNAL(valueChanged(int)), this, SLOT(sagSliceChanged(int)));
	connect(alphaSlider, SIGNAL(valueChanged(int)), this, SLOT(alphaChanged(int)));
}

/** 
*	This function loads the fixed image and sets the buffers for the three views
*
*/
void QRegistrationView::setFixedImage(InternalImageType::Pointer image)
{
	fixedImage = image;
	fixedSize = fixedImage->GetLargestPossibleRegion().GetSize();
	fixedSpacing = fixedImage->GetSpacing();
	
	//calculate the image min and max for windowing
	typedef itk::MinimumMaximumImageCalculator<InternalImageType> MinMaxCalcType;
	MinMaxCalcType::Pointer minMaxCalc = MinMaxCalcType::New();
	minMaxCalc->SetImage(image);
	minMaxCalc->SetRegion(image->GetLargestPossibleRegion());
	minMaxCalc->Compute();
	fixedImageMin = minMaxCalc->GetMinimum();
	fixedImageMax = minMaxCalc->GetMaximum();
		
	maxTransSlices = fixedSize[2];
	maxCorSlices = fixedSize[1];
	maxSagSlices = fixedSize[0];
	
	//center the sliders
	curTransSlice = (int) maxTransSlices / 2;
	curCorSlice = (int) maxCorSlices / 2;
	curSagSlice = (int) maxSagSlices / 2;
	
	transSliceSlider->setRange(0, maxTransSlices);
	corSliceSlider->setRange(0, maxCorSlices);
	sagSliceSlider->setRange(0, maxSagSlices);

	//delete all the old buffers, if they were previously used
	if(fixed_trans_slice_buffer != NULL)
	{
		delete[] fixed_trans_slice_buffer;
		fixed_trans_slice_buffer = NULL;
	}
	if(fixed_cor_slice_buffer != NULL)
	{
		delete[] fixed_cor_slice_buffer;
		fixed_cor_slice_buffer = NULL;
	}
	if(fixed_sag_slice_buffer != NULL)
	{
		delete[] fixed_sag_slice_buffer;
		fixed_sag_slice_buffer = NULL;
	}
	
	if(moving_trans_slice_buffer != NULL)
	{
		delete[] moving_trans_slice_buffer;
		moving_trans_slice_buffer = NULL;
	}
	if(moving_cor_slice_buffer != NULL)
	{
		delete[] moving_cor_slice_buffer;
		moving_cor_slice_buffer = NULL;
	}
	if(moving_sag_slice_buffer != NULL)
	{
		delete[] moving_sag_slice_buffer;
		moving_sag_slice_buffer = NULL;
	}

	//initialize buffers for moving image here, because moving image size is dependent upon fixed image size
	fixed_trans_slice_buffer = new unsigned char[fixedSize[0] * fixedSize[1]];
	fixed_cor_slice_buffer = new unsigned char[fixedSize[0] * fixedSize[2]];
	fixed_sag_slice_buffer = new unsigned char[fixedSize[1] * fixedSize[2]];
	
	moving_trans_slice_buffer = new unsigned char[fixedSize[0] * fixedSize[1]];
	moving_cor_slice_buffer = new unsigned char[fixedSize[0] * fixedSize[2]];
	moving_sag_slice_buffer = new unsigned char[fixedSize[1] * fixedSize[2]];

	//initialize the QImage parameters for 8 bit images
	transFixedImage = new QImage(fixed_trans_slice_buffer, fixedSize[0], fixedSize[1], QImage::Format_Indexed8);
	corFixedImage = new QImage(fixed_cor_slice_buffer, fixedSize[0], fixedSize[2], QImage::Format_Indexed8);
	sagFixedImage = new QImage(fixed_sag_slice_buffer, fixedSize[1], fixedSize[2], QImage::Format_Indexed8);
	
	transMovingImage = new QImage(moving_trans_slice_buffer, fixedSize[0], fixedSize[1], QImage::Format_Indexed8);
	corMovingImage = new QImage(moving_cor_slice_buffer, fixedSize[0], fixedSize[2], QImage::Format_Indexed8);
	sagMovingImage = new QImage(moving_sag_slice_buffer, fixedSize[1], fixedSize[2], QImage::Format_Indexed8);

	//set up the number of entries in the color table
	transFixedImage->setNumColors(256);
	corFixedImage->setNumColors(256);
	sagFixedImage->setNumColors(256);
	
	transMovingImage->setNumColors(256);
	corMovingImage->setNumColors(256);
	sagMovingImage->setNumColors(256);
	
	//now initilize the color table
	setFixedImageColorTable(fixedImageColorTable);
	setMovingImageColorTable(movingImageColorTable);
	
		//initialize the transform center if both images are loaded
	if(fixedImage.IsNotNull() && movingImage.IsNotNull() && display)
	{
		initializeTransform(transform, fixedImage, movingImage);

		//update the main display widget with the cenetered inital transform parameters
		display->setParameters(transform->GetParameters());
	}

	transSliceSlider->setSliderPosition(curTransSlice);
	corSliceSlider->setSliderPosition(curCorSlice);
	sagSliceSlider->setSliderPosition(curSagSlice);
	
	//notify all observers of the new image
	std::vector<ViewObserver *>::iterator it;
	for( it = m_observers.begin(); it < m_observers.end(); it++)
		(*it)->fixedImageChanged();

}

/**
*	This function loads the moving image and updates the viewer
*/
void QRegistrationView::setMovingImage(InternalImageType::Pointer image)
{
	movingImage = image;
	
	InternalImageType::SizeType movingSize = movingImage->GetLargestPossibleRegion().GetSize();
	InternalImageType::SpacingType movingSpacing = movingImage->GetSpacing();
	
	//calculate the image min and max for windowing
	typedef itk::MinimumMaximumImageCalculator<InternalImageType> MinMaxCalcType;
	MinMaxCalcType::Pointer minMaxCalc = MinMaxCalcType::New();
	minMaxCalc->SetImage(movingImage);
	minMaxCalc->SetRegion(movingImage->GetLargestPossibleRegion());
	minMaxCalc->Compute();
	movingImageMin = minMaxCalc->GetMinimum();
	movingImageMax = minMaxCalc->GetMaximum();
	
	//initialize the transform center if both images are loaded
	if(fixedImage.IsNotNull() && movingImage.IsNotNull() && display)
	{
		initializeTransform(transform, fixedImage, movingImage);

		//update the main display widget with the cenetered inital transform parameters
		display->setParameters(transform->GetParameters());
	}
	
	//notify all observers of the new image
	std::vector<ViewObserver *>::iterator it;
	for( it = m_observers.begin(); it < m_observers.end(); it++)
		(*it)->movingImageChanged();
	
	if(useWindowRegion)
		calculateMovingWindowRegion();
	
	updateView();
}

/**
*	This function is used to the the parameters for the transform applied to the moving image
*
*/
void QRegistrationView::setTransformParams(ParametersType params)
{		
	if(transform.IsNotNull())
		transform->SetParameters(params);
	
	//need to update the moving image window region
	calculateMovingWindowRegion();
	
	updateView();
}

/**
 *	This function updates the the requested slice view by first extracting the images from their itk::Image objects
 *	and then copying both images into their QImage buffers, and then applying an alpha mask to both if necessary
 *
 */
void QRegistrationView::updateSliceView(SliceView view)
{	
	QMutexLocker locker(&displayMutex);
	
	//Types necessary for the renering pipeline
	DisplayImageType::IndexType index;
	DisplayImageType::SizeType size;
	RegionType region;
	DisplayImageType::Pointer displayChecker;
	DisplayImageType::Pointer displayFixedSlice;
	DisplayImageType::Pointer displayMovingSlice;
	DisplayResamplerType::Pointer displayResampler;
	DisplayScalerType::Pointer displayScaler;
	int sideLength1, sideLength2;
	
	//various pixmaps needed for alpha blending and the display
	QPixmap movingPixmap;
	QPixmap movingAlpha;
	QPixmap resultPixmap;
	QImage* fixedQImage;
	QImage* movingQImage; 
	QLabel* displayLabel;
	
	//window region display
	QRect fixedWindowRect(-10, -10, 0, 0);
	QRect registrationVoiRect(-10, -10, 0, 0);
	//TransformType::InputPointType movingPoint, fixedPoint;
	bool drawFixedWindowRegion = false;
	bool drawRegistrationVoiRegion = false;
	
	//pointers to the image buffers
	unsigned char* src_buffer_fixed;
	unsigned char* src_buffer_moving;
	unsigned char* dest_buffer_fixed;
	unsigned char* dest_buffer_moving;
	
	//set the region information and pointers according to which slice should be updated
	switch (view) {
		case TransSlice:
			index[0] = 0;
			index[1] = 0;
			index[2] = curTransSlice;
			size[0] = fixedSize[0];
			size[1] = fixedSize[1];
			size[2] = 1;
			
			region.SetSize(size);
			region.SetIndex(index);
			dest_buffer_fixed = fixed_trans_slice_buffer;
			dest_buffer_moving = moving_trans_slice_buffer;
			fixedQImage = transFixedImage;
			movingQImage = transMovingImage;
			displayLabel = transViewLabel;
			
			sideLength1 = fixedSize[0];
			sideLength2 = fixedSize[1];
			
			if (showWinRegion && useWindowRegion && curTransSlice >= (unsigned int) fixedWindowRegion.GetIndex(2) &&
				curTransSlice <= fixedWindowRegion.GetIndex(2) + fixedWindowRegion.GetSize(2)) 
			{
				
				fixedWindowRect.setCoords(fixedWindowRegion.GetIndex(0), fixedWindowRegion.GetIndex(1), 
									  fixedWindowRegion.GetIndex(0) + fixedWindowRegion.GetSize(0),
									  fixedWindowRegion.GetIndex(1) + fixedWindowRegion.GetSize(1));
				drawFixedWindowRegion = true;
			}
			
			if(showRegistrationVoi && curTransSlice >= (unsigned int) voiRegion.GetIndex(2) &&
			   curTransSlice <= voiRegion.GetIndex(2) + voiRegion.GetSize(2))
			{
				registrationVoiRect.setCoords(voiRegion.GetIndex(0), voiRegion.GetIndex(1), 
										  voiRegion.GetIndex(0) + voiRegion.GetSize(0),
										  voiRegion.GetIndex(1) + voiRegion.GetSize(1));
				drawRegistrationVoiRegion = true;
				
			}
			
			break;
			
		case CorSlice:
			index[0] = 0;
			index[1] = curCorSlice;
			index[2] = 0;
			size[0] = fixedSize[0];
			size[1] = 1;
			size[2] = fixedSize[2];

			region.SetSize(size);
			region.SetIndex(index);
			dest_buffer_fixed = fixed_cor_slice_buffer;
			dest_buffer_moving = moving_cor_slice_buffer;
			fixedQImage = corFixedImage;
			movingQImage = corMovingImage;
			displayLabel = corViewLabel;
			
			sideLength1 = fixedSize[0];
			sideLength2 = fixedSize[2];

			if (showWinRegion && useWindowRegion && curCorSlice >= (unsigned int) fixedWindowRegion.GetIndex(1) &&
				curCorSlice <= fixedWindowRegion.GetIndex(1) + fixedWindowRegion.GetSize(1)) 
			{
			
				fixedWindowRect.setCoords(fixedWindowRegion.GetIndex(0), fixedWindowRegion.GetIndex(2), 
									  fixedWindowRegion.GetIndex(0) + fixedWindowRegion.GetSize(0),
									  fixedWindowRegion.GetIndex(2) + fixedWindowRegion.GetSize(2));
				drawFixedWindowRegion = true;
			}
			
			if (showRegistrationVoi && curCorSlice >= (unsigned int) voiRegion.GetIndex(1) &&
				curCorSlice <= voiRegion.GetIndex(1) + voiRegion.GetSize(1)) 
			{
				
				registrationVoiRect.setCoords(voiRegion.GetIndex(0), voiRegion.GetIndex(2), 
										  voiRegion.GetIndex(0) + voiRegion.GetSize(0),
										  voiRegion.GetIndex(2) + voiRegion.GetSize(2));
				drawRegistrationVoiRegion = true;
			}

			break;
			
		case SagSlice:
			index[0] = curSagSlice;
			index[1] = 0;
			index[2] = 0;
			size[0] = 1;
			size[1] = fixedSize[1];
			size[2] = fixedSize[2];
						
			region.SetSize(size);
			region.SetIndex(index);
			dest_buffer_fixed = fixed_sag_slice_buffer;
			dest_buffer_moving = moving_sag_slice_buffer;
			fixedQImage = sagFixedImage;
			movingQImage = sagMovingImage;
			displayLabel = sagViewLabel;
			
			sideLength1 = fixedSize[1];
			sideLength2 = fixedSize[2];
			
			if (showWinRegion && useWindowRegion && curSagSlice >= (unsigned int) fixedWindowRegion.GetIndex(0) &&
				curSagSlice <= fixedWindowRegion.GetIndex(0) + fixedWindowRegion.GetSize(0)) 
			{
				
				fixedWindowRect.setCoords(fixedWindowRegion.GetIndex(1), fixedWindowRegion.GetIndex(2), 
									  fixedWindowRegion.GetIndex(1) + fixedWindowRegion.GetSize(1),
									  fixedWindowRegion.GetIndex(2) + fixedWindowRegion.GetSize(2));
				drawFixedWindowRegion = true;
			}

			if (showRegistrationVoi && curSagSlice >= (unsigned int) voiRegion.GetIndex(0) &&
				curSagSlice <= voiRegion.GetIndex(0) + voiRegion.GetSize(0)) 
			{
				
				registrationVoiRect.setCoords(voiRegion.GetIndex(1), voiRegion.GetIndex(2), 
										  voiRegion.GetIndex(1) + voiRegion.GetSize(1),
										  voiRegion.GetIndex(2) + voiRegion.GetSize(2));
				drawRegistrationVoiRegion = true;
			}
			
			break;
			
		default:
			return;
			break;
	}
	
	//show the two images as a checkerboard composite if so wished by user
	if(showCheckered && fixedImage && movingImage)
	{
		//first set up the checkerboard filter and set the pattern
		CheckerBoardType::Pointer checker = CheckerBoardType::New();
		checker->SetCheckerPattern(checkerPattern);
		
		//window and resacle the fixed image to fit in the color table
		//See Issue #9 for explaination of the windowing formula
		displayScaler = DisplayScalerType::New();
		displayScaler->SetInput(fixedImage);
		displayScaler->SetShift( - winMinFixed * (fixedImageMax - fixedImageMin) - fixedImageMin);
		displayScaler->SetScale( 255 / (  winMaxFixed * (fixedImageMax - fixedImageMin) - winMinFixed * (fixedImageMax - fixedImageMin)) );
		
		//allow the user to select the image order
		if(checkerFixedFirst)
			checker->SetInput1(displayScaler->GetOutput());
		else
			checker->SetInput2(displayScaler->GetOutput());
		
		//resample the resacled data
		displayResampler = DisplayResamplerType::New();
		displayResampler->SetInput(movingImage);
		displayResampler->SetSize(fixedSize);
		displayResampler->SetOutputOrigin(fixedImage->GetOrigin());
		displayResampler->SetOutputSpacing(fixedSpacing);
		displayResampler->SetDefaultPixelValue(winMinMoving * (movingImageMax - movingImageMin) + movingImageMin);
		displayResampler->SetTransform(transform);
		
		//now rescale the moving image as well
		//See Issue #9 for explaination of the windowing formula
		DisplayScalerType::Pointer displayScaler2 = DisplayScalerType::New();
		displayScaler2->SetInput(displayResampler->GetOutput());
		displayScaler2->SetShift( - winMinMoving * (movingImageMax - movingImageMin) - movingImageMin);
		displayScaler2->SetScale( 255 / (  winMaxMoving * (movingImageMax - movingImageMin) - winMinMoving * (movingImageMax - movingImageMin)) );
		
		if(checkerFixedFirst)
			checker->SetInput2(displayScaler2->GetOutput());
		else
			checker->SetInput1(displayScaler2->GetOutput());
		
		//set the requested slice
		//Note: this region is back propogated to all filters in the pipeline
		//so only the currently requested slice is processed and not the whole image
		displayChecker = checker->GetOutput();
		displayChecker->SetRequestedRegion(region);
		displayChecker->SetBufferedRegion(region);
		
		try 
		{ 
			displayChecker->Update(); 
		} 
		catch( itk::ExceptionObject & err ) 
		{ 
			logging::logError("Error rescaling fixed image in the trans view registration widget!!!"); 
			std::cerr << err << std::endl; 
			return;
		} 
				
		//copy region into the display buffer
		src_buffer_fixed = displayChecker->GetBufferPointer();
		memcpy(dest_buffer_fixed, src_buffer_fixed, sideLength1 * sideLength2);
		
		//convert the fixed image to a pixmap and resize
		if(interpolationMode == 1){
			displayLabel->setPixmap(QPixmap::fromImage(fixedQImage->scaled(sideLength1 * zoomFactor, sideLength2 * zoomFactor)));
		}
		else
		{
			displayLabel->setPixmap(QPixmap::fromImage(fixedQImage->scaled(sideLength1 * zoomFactor, sideLength2 * zoomFactor, 
																		   Qt::KeepAspectRatio, Qt::SmoothTransformation)));
		}
				
	}		//otherwise show the images blended on top of each other
	else if(fixedImage)
	{
		
		//first rescale the fixed image
		//See Issue #9 for explaination of the windowing formula
		displayScaler = DisplayScalerType::New();
		displayScaler->SetInput(fixedImage);
		displayScaler->SetShift( - winMinFixed * (fixedImageMax - fixedImageMin) - fixedImageMin );
		displayScaler->SetScale( 255 / (  winMaxFixed * (fixedImageMax - fixedImageMin) - winMinFixed * (fixedImageMax - fixedImageMin)) );
		
		//select the region (slice) to be copied into the image buffer
		displayFixedSlice = displayScaler->GetOutput();
		displayFixedSlice->SetRequestedRegion(region);
		displayFixedSlice->SetBufferedRegion(region);
		
		try 
		{ 
			displayFixedSlice->Update(); 
		} 
		catch( itk::ExceptionObject & err ) 
		{ 
			std::cerr << "Error rescaling fixed image in the trans view registration widget!!!" << std::endl; 
			std::cerr << err << std::endl; 
			return;
		} 
		
		//copy region into the display buffer
		src_buffer_fixed = displayFixedSlice->GetBufferPointer();
		memcpy(dest_buffer_fixed, src_buffer_fixed, sideLength1 * sideLength2);	
		
		//only show the moving image if the fixed image was loaded
		//because the moving image has to be resampled into the moving images space
		//and because we also have to alpha blend the two images together, otherwise the result image is too weak
		if(movingImage)
		{			
			//first resmaple the moving image into the fixed Images space
			//the resampler has to be the first filter in the pipeline otherwise it causes
			//the predecessor filter to operate on the entire image region not just the slice
			displayResampler = DisplayResamplerType::New();
			displayResampler->SetInput(movingImage);
			displayResampler->SetSize(fixedSize);
			displayResampler->SetOutputOrigin(fixedImage->GetOrigin());
			displayResampler->SetOutputSpacing(fixedSpacing);
			displayResampler->SetDefaultPixelValue(winMinMoving * (movingImageMax - movingImageMin) + movingImageMin);
			displayResampler->SetTransform(transform);
			
			//now rescale the resampled data
			//See Issue #9 for explaination of the windowing formula
			displayScaler = DisplayScalerType::New();
			displayScaler->SetInput(displayResampler->GetOutput());
			displayScaler->SetShift( - winMinMoving * (movingImageMax - movingImageMin) - movingImageMin);
			displayScaler->SetScale( 255 / (  winMaxMoving * (movingImageMax - movingImageMin) - winMinMoving * (movingImageMax - movingImageMin)) );
			
			//Extract the required slice
			displayMovingSlice = displayScaler->GetOutput();
			displayMovingSlice->SetRequestedRegion(region);
			displayMovingSlice->SetBufferedRegion(region);
			
			try 
			{ 
				displayMovingSlice->Update(); 
			} 
			catch( itk::ExceptionObject & err ) 
			{ 
				logging::logError("Error resampling or rescaling moving image in the trans view registration widget!!!"); 
				std::cerr << err << std::endl; 
				return;
			} 
			
			//copy data from region pointer to the display buffer for the moving image
			src_buffer_moving = displayMovingSlice->GetBufferPointer();
			memcpy(dest_buffer_moving, src_buffer_moving, sideLength1 * sideLength2);
						
			//make a pixmap of the moving image with the right alpha value
			movingPixmap = QPixmap::fromImage(*movingQImage);
			movingAlpha = movingPixmap;
			QPainter pMoving(&movingAlpha);
			pMoving.fillRect(movingAlpha.rect(), QColor(255 - alpha, 255 - alpha, 255 - alpha));
			pMoving.end();
			movingPixmap.setAlphaChannel(movingAlpha);
			
			//create a result pixmap and paint the two alpha blended images on top of it
			QPoint origin = QPoint(0, 0);
			resultPixmap = QPixmap(fixedQImage->size());
			QPainter pResult(&resultPixmap);
			pResult.fillRect(QRect(0, 0, sideLength1, sideLength2), Qt::black);
			pResult.drawImage(origin, *fixedQImage);
			pResult.drawPixmap(0, 0, movingPixmap);
			
			if(drawFixedWindowRegion)
			{
				pResult.setPen(QColor(255, 153, 0));
				pResult.drawRect(fixedWindowRect);
			}
			
			if(drawRegistrationVoiRegion)
			{
				pResult.setPen(QColor(0, 230, 0));
				pResult.drawRect(registrationVoiRect);
			}
			
			pResult.end();
			
			//resize the result pixmap and add it the widget display
			if(interpolationMode == 1){
				displayLabel->setPixmap(resultPixmap.scaled(sideLength1 * zoomFactor, sideLength2 * zoomFactor));
			}
			else
			{
				displayLabel->setPixmap(resultPixmap.scaled(sideLength1 * zoomFactor, sideLength2 * zoomFactor, Qt::KeepAspectRatio, Qt::SmoothTransformation));
			}
		}
		else
		{	
			resultPixmap = QPixmap::fromImage(*fixedQImage);
			QPainter display(&resultPixmap);
			
			if(drawFixedWindowRegion)
			{
				display.setPen(QColor(255, 153, 0));
				display.drawRect(fixedWindowRect);
			}
			
			if(drawRegistrationVoiRegion)
			{
				display.setPen(QColor(0, 230, 0));
				display.drawRect(registrationVoiRect);
			}
			
			display.end();

			//other simply convert the fixed image to a pixmap and resize
			if(interpolationMode == 1){
				displayLabel->setPixmap(resultPixmap.scaled(sideLength1 * zoomFactor, sideLength2 * zoomFactor));
			}
			else
			{
				displayLabel->setPixmap(resultPixmap.scaled(sideLength1 * zoomFactor, sideLength2 * zoomFactor,
																			   Qt::KeepAspectRatio, Qt::SmoothTransformation));
			}
		}
	}
}

void QRegistrationView::transSliceChanged(int sliceNr)
{
	if(0 <= sliceNr &&  (unsigned int) sliceNr  < maxTransSlices)
	{
		curTransSlice = sliceNr;
		transSliceLabel->setNum(sliceNr);
		updateSliceView(TransSlice);
	}
}


void QRegistrationView::corSliceChanged(int sliceNr)
{	

	if (0 <= sliceNr && (unsigned int) sliceNr < maxCorSlices) 
	{
		curCorSlice = sliceNr;
		corSliceLabel->setNum(sliceNr);
		updateSliceView(CorSlice);
	
	}
}

void QRegistrationView::sagSliceChanged(int sliceNr)
{
	if(0 <= sliceNr && (unsigned int) sliceNr < maxSagSlices)
	{
		curSagSlice = sliceNr;
		sagSliceLabel->setNum(sliceNr);
		updateSliceView(SagSlice);
	}
}

/**
 *	This function is used by the Interface to set the alpha blending value
 *
 */
void QRegistrationView::alphaChanged(int val)
{
	alpha = val;
	alphaLabel->setNum( (int) (alpha / 2.55));
	updateView();
}

void QRegistrationView::updateView()
{	
	updateSliceView(TransSlice);
	updateSliceView(CorSlice);
	updateSliceView(SagSlice);
}

/**
 *	This function sets a black white linear color table for the input image
 *
 */
void QRegistrationView::setBlackWhiteColorTable(QImage *image)
{
	for(int i = 0; i < 256; i++)
		image->setColor(i, qRgb(i, i , i));
}

/**
 *	This function sets a linear red color table
 *
 */
void QRegistrationView::setRedColorTable(QImage *image)
{
	for(int i = 0; i < 256; i++)
		image->setColor(i, qRgb(i, 0 , 0));
}

/**
 *	This function sets a red white gradient based color table
 *
 */
void QRegistrationView::setHotRedColorTable(QImage *image)
{
	char r[256] =  { 0, 1, 2, 4, 5, 7, 8, 10, 11, 13, 14, 15, 17, 18, 20, 21, 23, 24, 26, 27, 28, 30, 31,
					33, 34, 36, 37, 39, 40, 42, 43, 44, 46, 47, 49, 50, 52, 53, 55, 56, 57, 59, 60, 62, 63,
					65, 66, 68, 69, 70, 72, 73, 75, 76, 78, 79, 81, 82, 84, 85, 86, 88, 89, 91, 92, 94, 95,
					97, 98, 99, 101, 102, 104, 105, 107, 108, 110, 111, 113, 114, 115, 117, 118, 120, 121, 123,
					124, 126, 127, 128, 130, 131, 133, 134, 136, 137, 139, 140, 141, 143, 144, 146, 147, 149,
					150, 152, 153, 155, 156, 157, 159, 160, 162, 163, 165, 166, 168, 169, 170, 172, 173, 175,
					176, 178, 179, 181, 182, 184, 185, 186, 188, 189, 191, 192, 194, 195, 197, 198, 199, 201,
					202, 204, 205, 207, 208, 210, 211, 212, 214, 215, 217, 218, 220, 221, 223, 224, 226, 227,
					228, 230, 231, 233, 234, 236, 237, 239, 240, 241, 243, 244, 246, 247, 249, 250, 252, 253,
					255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
					255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
					255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
					255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 
					255, 255, 255, 255, 255, 255, 255, 255 };
				
	char g[256] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
					0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
					0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
					0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
					0, 0, 0, 0, 0, 1, 3, 5, 7, 9, 11, 13, 15, 17, 18, 20, 22, 24, 26, 28, 30, 32, 34, 35, 37,
					39, 41, 43, 45, 47, 49, 51, 52, 54, 56, 58, 60, 62, 64, 66, 68, 69, 71, 73, 75, 77, 79, 81,
					83, 85, 86, 88, 90, 92, 94, 96, 98, 100, 102, 103, 105, 107, 109, 111, 113, 115, 117, 119,
					120, 122, 124, 126, 128, 130, 132, 134, 136, 137, 139, 141, 143, 145, 147, 149, 151, 153,
					154, 156, 158, 160, 162, 164, 166, 168, 170, 171, 173, 175, 177, 179, 181, 183, 185, 187, 
					188, 190, 192, 194, 196, 198, 200, 202, 204, 205, 207, 209, 211, 213, 215, 217, 219, 221,
					222, 224, 226, 228, 230, 232, 234, 236, 238, 239, 241, 243, 245, 247, 249, 251, 253, 255 };
					
	char b[256] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
					0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
					0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
					0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
					0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
					0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
					0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 7, 11, 15, 19, 23, 27, 31, 35, 39, 43, 47, 51, 54, 58,
					62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 105, 109, 113, 117, 121, 125, 129, 133, 137,
					141, 145, 149, 153, 156, 160, 164, 168, 172, 176, 180, 184, 188, 192, 196, 200, 204, 207,
					211, 215, 219, 223, 227, 231, 235, 239, 243, 247, 251, 255 };
					
	for(int i = 0; i < 256; i++)
		image->setColor(i, qRgb(r[i], g[i] , b[i]));

}

/**
 *	This function sets a gradient based blue white color table for the input image
 *
 */
void QRegistrationView::setBlueWhiteColorTable(QImage* image)
{

	char r[256] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
					0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
					0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
					0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
					0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
					0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
					0, 0, 0, 0, 0, 0, 0, 5, 9, 13, 17, 21, 25, 29, 33, 37, 41, 45, 49, 53, 57, 61, 65, 69, 73, 77,
					81, 85, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 134, 138, 142, 146, 150, 154, 158,
					162, 166, 170, 175, 179, 183, 187, 191, 195, 199, 203, 207, 211, 215, 219, 223, 227, 231,
					235, 239, 243, 247, 251, 255 };
					
	char g[256] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
					0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
					0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
					0, 2, 4, 5, 7, 9, 10, 12, 13, 15, 17, 18, 20, 21, 23, 25, 26, 28, 29, 31, 33, 34, 36, 37, 39, 41,
					42, 44, 45, 47, 49, 50, 52, 53, 55, 57, 58, 60, 61, 63, 65, 66, 68, 69, 71, 73, 74, 76, 77, 79, 81,
					82, 84, 85, 87, 89, 90, 92, 94, 95, 97, 98, 100, 102, 103, 105, 106, 108, 110, 111, 113, 114, 116,
					118, 119, 121, 122, 124, 126, 127, 129, 130, 132, 134, 135, 137, 138, 140, 142, 143, 145, 146, 148,
					150, 151, 153, 154, 156, 158, 159, 161, 162, 164, 166, 167, 169, 170, 172, 174, 175, 177, 179, 180,
					182, 183, 185, 187, 188, 190, 191, 193, 195, 196, 198, 199, 201, 203, 204, 206, 207, 209, 211, 212,
					214, 215, 217, 219, 220, 222, 223, 225, 227, 228, 230, 231, 233, 235, 236, 238, 239, 241, 243, 244,
					246, 247, 249, 251, 252, 254, 255 };
					
	char b[256] = { 0, 1, 2, 4, 5, 6, 8, 9, 10, 12, 13, 14, 16, 17, 18, 20, 21, 23, 24, 25, 27, 28, 29, 31, 32, 33, 35,
					36, 37, 39, 40, 42, 43, 44, 46, 47, 48, 50, 51, 52, 54, 55, 56, 58, 59, 61, 62, 63, 65, 66, 67, 69,
					70, 71, 73, 74, 75, 77, 78, 80, 81, 82, 84, 85, 86, 88, 89, 90, 92, 93, 94, 96, 97, 99, 100, 101,
					103, 104, 105, 107, 108, 109, 111, 112, 113, 115, 116, 118, 119, 120, 122, 123, 124, 126, 127, 128,
					130, 131, 132, 134, 135, 136, 138, 139, 141, 142, 143, 145, 146, 147, 149, 150, 151, 153, 154, 155,
					157, 158, 160, 161, 162, 164, 165, 166, 168, 169, 170, 172, 173, 174, 176, 177, 179, 180, 181, 183,
					184, 185, 187, 188, 189, 191, 192, 193, 195, 196, 198, 199, 200, 202, 203, 204, 206, 207, 208, 210,
					211, 212, 214, 215, 217, 218, 219, 221, 222, 223, 225, 226, 227, 229, 230, 231, 233, 234, 236, 237,
					238, 240, 241, 242, 244, 245, 246, 248, 249, 250, 252, 253, 255, 255, 255, 255, 255, 255, 255, 255,
					255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
					255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
					255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255 };


	for(int i = 0; i < 256; i++)
		image->setColor(i, qRgb(r[i], g[i] , b[i]));

}

/**
 *	This function sets the zoom factor used by the display QPixmap object
 *
 */
void QRegistrationView::setZoomFactor(int factor)
{
	zoomFactor = factor;
	updateView();
}

/**
 *	This function sets the interpolation mode used to rescale the QPixmap object
 *
 */
void QRegistrationView::setInterpolationMode(int mode)
{
	interpolationMode = mode;
	updateView();
}


/**
 *	This function is called by the interface to set the display into chekcerboard mode
 *
 */
void QRegistrationView::showCheckeredImage(bool show)
{
	showCheckered = show;
	updateView();
}

/**
 *	Sets the color table for the fixed image
 */
void QRegistrationView::setFixedImageColorTable(ColorTable table)
{
	fixedImageColorTable = table;
	if(fixedImage.IsNotNull())
	{
		switch (fixedImageColorTable) {
			case HotRed:
			default:
				setHotRedColorTable(transFixedImage);
				setHotRedColorTable(corFixedImage);
				setHotRedColorTable(sagFixedImage);
				break;
				
			case HotBlue:
				setBlueWhiteColorTable(transFixedImage);
				setBlueWhiteColorTable(corFixedImage);
				setBlueWhiteColorTable(sagFixedImage);
				break;
				
			case LinearRed:
				setRedColorTable(transFixedImage);
				setRedColorTable(corFixedImage);
				setRedColorTable(sagFixedImage);
				break;
				
			case BlackWhite:
				setBlackWhiteColorTable(transFixedImage);
				setBlackWhiteColorTable(corFixedImage);
				setBlackWhiteColorTable(sagFixedImage);
				break;
		}
		
		updateView();
	}
}

/**
 *	Sets the color table for the fixed image
 */
void QRegistrationView::setMovingImageColorTable(ColorTable table)
{
	movingImageColorTable = table;
	if(fixedImage.IsNotNull())
	{
		switch (movingImageColorTable) {
			case HotRed:
				setHotRedColorTable(transMovingImage);
				setHotRedColorTable(corMovingImage);
				setHotRedColorTable(sagMovingImage);
				break;
				
			case HotBlue:
			default:
				setBlueWhiteColorTable(transMovingImage);
				setBlueWhiteColorTable(corMovingImage);
				setBlueWhiteColorTable(sagMovingImage);
				break;
			
			case LinearRed:
				setRedColorTable(transMovingImage);
				setRedColorTable(corMovingImage);
				setRedColorTable(sagMovingImage);
				break;
			
			case BlackWhite:
				setBlackWhiteColorTable(transMovingImage);
				setBlackWhiteColorTable(corMovingImage);
				setBlackWhiteColorTable(sagMovingImage);
				break;
		}
	
		updateView();
	}
}

ColorTable QRegistrationView::getFixedImageColorTable()
{
	return fixedImageColorTable;
}

ColorTable QRegistrationView::getMovingImageColorTable()
{
	return movingImageColorTable;
}

InternalImageType::Pointer QRegistrationView::getFixedImage()
{
	return fixedImage;
}

InternalImageType::Pointer QRegistrationView::getMovingImage()
{
	return movingImage;
}

float QRegistrationView::getFixedWindowMin()
{
	return winMinFixed;
}

void QRegistrationView::setFixedWindowMin(float val)
{
	winMinFixed = val;
}

float QRegistrationView::getFixedWindowMax()
{
	return winMaxFixed;
}

void QRegistrationView::setFixedWindowMax(float val)
{
	winMaxFixed = val;
}

float QRegistrationView::getMovingWindowMin()
{
	return winMinMoving;
}

void QRegistrationView::setMovingWindowMin(float val)
{
	winMinMoving = val;
}

float QRegistrationView::getMovingWindowMax()
{
	return winMaxMoving;
}

void QRegistrationView::setMovingWindowMax(float val)
{
	winMaxMoving = val;
}

bool QRegistrationView::isFixedWindowMaxMode()
{
	return winMaxModeFixed;
}

void QRegistrationView::setFixedWindowMaxMode(bool setMaxMode)
{
	winMaxModeFixed = setMaxMode;
}

bool QRegistrationView::isMovingWindowMaxMode()
{
	return winMaxModeMoving;
}

void QRegistrationView::setMovingWindowMaxMode(bool setMaxMode)
{
	winMaxModeMoving = setMaxMode;
}

InternalPixelType QRegistrationView::getFixedImageMin()
{
	return fixedImageMin;
}

InternalPixelType QRegistrationView::getFixedImageMax()
{
	return fixedImageMax;
}

InternalPixelType QRegistrationView::getMovingImageMin()
{
	return movingImageMin;
}

InternalPixelType QRegistrationView::getMovingImageMax()
{
	return movingImageMax;
}

void QRegistrationView::setCheckerBoardOrder(bool fixedFirst)
{
	checkerFixedFirst = fixedFirst;
	updateView();
}

bool QRegistrationView::getCheckerBoardOrder()
{
	return checkerFixedFirst;
}

CheckerBoardType::PatternArrayType QRegistrationView::getCheckerBoardPattern()
{
	return checkerPattern;
}

void QRegistrationView::setCheckerBoardPattern(CheckerBoardType::PatternArrayType pattern)
{
	checkerPattern = pattern;
}

void QRegistrationView::addObserver(ViewObserver* observer)
{
	m_observers.push_back(observer);
}

void QRegistrationView::removeObserver(ViewObserver* observer)
{
	std::vector<ViewObserver *>::iterator it;
	for( it = m_observers.begin(); it < m_observers.end(); it++)
		if(*it == observer)
			m_observers.erase(it);
}

void QRegistrationView::setFixedWindowRegion(RegionType region)
{	
	fixedWindowRegion = region;
}

void QRegistrationView::calculateFixedWindowRegion()
{
	if(fixedImage.IsNull())
		return;
	
	//verify the bounds of the region, truncate if necessary
	RegionType fixedImageRegion = fixedImage->GetLargestPossibleRegion();
	if(fixedWindowRegion.GetIndex(0) < fixedImageRegion.GetIndex(0))
		fixedWindowRegion.SetIndex(0, fixedImageRegion.GetIndex(0));
	else if(fixedWindowRegion.GetIndex(0) >  fixedImageRegion.GetIndex(0) + (signed int) fixedImageRegion.GetSize(0))
		fixedWindowRegion.SetIndex(0, fixedImageRegion.GetIndex(0) + fixedImageRegion.GetSize(0));
	else if(fixedWindowRegion.GetIndex(0) + fixedWindowRegion.GetSize(0) >
			fixedImageRegion.GetIndex(0) + fixedImageRegion.GetSize(0))
		fixedWindowRegion.SetSize(0, fixedWindowRegion.GetIndex(0) - fixedImageRegion.GetIndex(0) + fixedImageRegion.GetSize(0));
	
	if(fixedWindowRegion.GetIndex(1) < fixedImageRegion.GetIndex(1))
		fixedWindowRegion.SetIndex(1, fixedImageRegion.GetIndex(1));
	else if( fixedWindowRegion.GetIndex(1) > fixedImageRegion.GetIndex(1) + (signed int) fixedImageRegion.GetSize(1))
		fixedWindowRegion.SetIndex(1, fixedImageRegion.GetIndex(1) + fixedImageRegion.GetSize(1));
	else if(fixedWindowRegion.GetIndex(1) + fixedWindowRegion.GetSize(1) >
			fixedImageRegion.GetIndex(1) + fixedImageRegion.GetSize(1))
		fixedWindowRegion.SetSize(1, fixedWindowRegion.GetIndex(1) - fixedImageRegion.GetIndex(1) + fixedImageRegion.GetSize(1));
	
	if(fixedWindowRegion.GetIndex(2) < fixedImageRegion.GetIndex(2))
		fixedWindowRegion.SetIndex(2, fixedImageRegion.GetIndex(2));
	else if(fixedWindowRegion.GetIndex(2) > fixedImageRegion.GetIndex(2) + (signed int) fixedImageRegion.GetSize(2))
		fixedWindowRegion.SetIndex(2, fixedImageRegion.GetIndex(2) + fixedImageRegion.GetSize(2));
	else if(fixedWindowRegion.GetIndex(2) + fixedWindowRegion.GetSize(2) >
			fixedImageRegion.GetIndex(2) + fixedImageRegion.GetSize(2))
		fixedWindowRegion.SetSize(2, fixedWindowRegion.GetIndex(2) - fixedImageRegion.GetIndex(2) + fixedImageRegion.GetSize(2));
	
	//calculate the image min and max for windowing
	typedef itk::MinimumMaximumImageCalculator<InternalImageType> MinMaxCalcType;
	MinMaxCalcType::Pointer minMaxCalc = MinMaxCalcType::New();
	minMaxCalc->SetImage(fixedImage);
	minMaxCalc->SetRegion(fixedWindowRegion);
	minMaxCalc->Compute();
	InternalPixelType min = minMaxCalc->GetMinimum();
	InternalPixelType max = minMaxCalc->GetMaximum();
	
	winMaxFixed = (max - fixedImageMin) / (fixedImageMax - fixedImageMin);
	winMinFixed = (min - fixedImageMin) / (fixedImageMax - fixedImageMin);
	
	useWindowRegion = true;
}

void QRegistrationView::calculateMovingWindowRegion()
{	
	if(!useWindowRegion || movingImage.IsNull() || fixedImage.IsNull())
		return;
	
	//first need to resample the moving image, so that we get the correct region
	DisplayResamplerType::Pointer resampler = DisplayResamplerType::New();
	resampler->SetInput(movingImage);
	resampler->SetSize(fixedWindowRegion.GetSize());
	resampler->SetOutputOrigin(fixedImage->GetOrigin());
	resampler->SetOutputStartIndex(fixedWindowRegion.GetIndex());
	resampler->SetOutputSpacing(fixedSpacing);
	resampler->SetDefaultPixelValue(movingImageMin);
	resampler->SetTransform(transform);
	
	InternalImageType::Pointer regionImage = InternalImageType::New();
	regionImage->SetBufferedRegion(fixedWindowRegion);
	regionImage->SetRequestedRegion(fixedWindowRegion);
	regionImage = resampler->GetOutput();
	regionImage->Update();

	//calculate the image min and max for windowing
	typedef itk::MinimumMaximumImageCalculator<InternalImageType> MinMaxCalcType;
	MinMaxCalcType::Pointer minMaxCalc = MinMaxCalcType::New();
	minMaxCalc->SetImage(regionImage);
	minMaxCalc->SetRegion(fixedWindowRegion);
	minMaxCalc->Compute();
	InternalPixelType min = minMaxCalc->GetMinimum();
	InternalPixelType max = minMaxCalc->GetMaximum();
	
	winMaxMoving = (max - movingImageMin) / (movingImageMax - movingImageMin);
	winMinMoving = (min - movingImageMin) / (movingImageMax - movingImageMin);
}

RegionType QRegistrationView::getFixedWindowRegion()
{
	return fixedWindowRegion;
}

void QRegistrationView::setWindowingMode(bool regionMode)
{
	useWindowRegion = regionMode;
	if (useWindowRegion)
	{
		calculateFixedWindowRegion();
		calculateMovingWindowRegion();
	}	
}

bool QRegistrationView::getWindowingMode()
{
	return useWindowRegion;
}

void QRegistrationView::showWindowRegion(bool show)
{
	showWinRegion = show;
}

void QRegistrationView::setVoiRegion(RegionType region)
{
	voiRegion = region;
}

RegionType QRegistrationView::getVoiRegion()
{
	return voiRegion;
}

bool QRegistrationView::isShowingVoi()
{
	return showRegistrationVoi;
}

void QRegistrationView::showVoi(bool show)
{
	showRegistrationVoi = show;
}


