//
//  $Id$  
//  $Lastlog$
//
//  CoreTest.cpp
//  petspectfusionTest
//
//	Created 27/10/2009, core unit test cases.
//  Copyright (c) 2007 - 2009 Brian Jensen. All rights reserved.
//

#include <fstream>
#include <iostream>
#include <math.h>
#include <sstream>

#include "CoreTest.h"
#include "PetSpectFusionApp.h"
#include "PetSpectFusionViewer.h"
#include "Registration.h"

//Allow versor to euler conversion to vary by this amount
#define EULER_TOLERANCE 1e-1

//Versors should be accurate to 7 places after the comma (slightly less than max mantissa size with a 32bit float)
#define VERSOR_TOLERANCE 1e-7

// Add the core test suite to the test registry
CPPUNIT_TEST_SUITE_REGISTRATION( CoreTest );

void CoreTest::setUp()
{

}

void CoreTest::tearDown()
{

}

void CoreTest::eulerConversionTest()
{
	RotationType eulerAngles(3), versor;
	
	//first test the default configuration
	eulerAngles[0] = 0.0;
	eulerAngles[1] = 0.0;
	eulerAngles[2] = 0.0;
	
	versor = convertEulerToVersor(eulerAngles);
	
	CPPUNIT_ASSERT( fabs(0.0 - versor[0]) <= VERSOR_TOLERANCE);
	CPPUNIT_ASSERT( fabs(0.0 - versor[1]) <= VERSOR_TOLERANCE);
	CPPUNIT_ASSERT( fabs(0.0 - versor[2]) <= VERSOR_TOLERANCE);
	
	//then test a simple rotation and verify the versor is correct
	eulerAngles[0] = 30.0;
	eulerAngles[1] = 30.0;
	eulerAngles[2] = 0.0;
	
	versor = convertEulerToVersor(eulerAngles);
		
	CPPUNIT_ASSERT( fabs(0.25 - versor[0]) <= VERSOR_TOLERANCE);
	CPPUNIT_ASSERT( fabs(0.25 - versor[1]) <= VERSOR_TOLERANCE);
	CPPUNIT_ASSERT( fabs(-0.0669872984 - versor[2]) <= VERSOR_TOLERANCE );
	
	//now verify a more complex rotation
	eulerAngles[0] = 90.0;
	eulerAngles[1] = 45.0;
	eulerAngles[2] = 30.0;
	
	versor = convertEulerToVersor(eulerAngles);
	
	CPPUNIT_ASSERT( fabs(0.5609855 - versor[0]) <= VERSOR_TOLERANCE);
	CPPUNIT_ASSERT( fabs(0.4304593 - versor[1]) <= VERSOR_TOLERANCE);
	CPPUNIT_ASSERT( fabs(-0.0922960 - versor[2]) <= VERSOR_TOLERANCE);
}

void CoreTest::versorConversionTest()
{
	RotationType versor(3), eulerAngles;
	
	//first test the default configuration, 
	versor[0] = 0.0;
	versor[1] = 0.0;
	versor[2] = 0.0;
	
	eulerAngles = convertVersorToEuler(versor);
	
	CPPUNIT_ASSERT( fabs(0.0 - eulerAngles[0]) <= EULER_TOLERANCE);
	CPPUNIT_ASSERT( fabs(0.0 - eulerAngles[1]) <= EULER_TOLERANCE);
	CPPUNIT_ASSERT( fabs(0.0 - eulerAngles[2]) <= EULER_TOLERANCE);
	
	//Versor represents 30 30 0  around X Y Z respectively
	versor[0] = 0.25;
	versor[1] = 0.25;
	versor[2] = -0.066987;
	
	eulerAngles = convertVersorToEuler(versor);
	
	CPPUNIT_ASSERT( fabs(30.0 - eulerAngles[0]) <= EULER_TOLERANCE);
	CPPUNIT_ASSERT( fabs(30.0 - eulerAngles[1]) <= EULER_TOLERANCE);
	CPPUNIT_ASSERT( fabs(0.0 - eulerAngles[2]) <= EULER_TOLERANCE);
	
	//now a more complex conversion, 90 45 30
	versor[0] = 0.5609855;
	versor[1] = 0.4304593;
	versor[2] = -0.0922960;

	eulerAngles = convertVersorToEuler(versor);
	
	CPPUNIT_ASSERT( fabs(90.0 - eulerAngles[0]) <= EULER_TOLERANCE);
	CPPUNIT_ASSERT( fabs(45.0 - eulerAngles[1]) <= EULER_TOLERANCE);
	CPPUNIT_ASSERT( fabs(30.0 - eulerAngles[2]) <= EULER_TOLERANCE);
}

void CoreTest::combinedConversionTest()
{
	RotationType eulerAngles(3);
	
	//Test a complicated Euler angles configuration, close to a singularity
	eulerAngles[0] = 165.0;
	eulerAngles[1] = -85.0;
	eulerAngles[2] = 15.0;
	
	//convert to versor and then convert back
	RotationType versor = convertEulerToVersor(eulerAngles);
	RotationType eulerConverted = convertVersorToEuler(versor);
	
	//make sure we ended up where we started from
	CPPUNIT_ASSERT( fabs(eulerAngles[0] - eulerConverted[0]) < EULER_TOLERANCE);
	CPPUNIT_ASSERT( fabs(eulerAngles[1] - eulerConverted[1]) < EULER_TOLERANCE);
	CPPUNIT_ASSERT( fabs(eulerAngles[2] - eulerConverted[2]) < EULER_TOLERANCE);
	
	//perform the same rotation, this time checking versor consistency
	versor[0] = -0.1828385;
	versor[1] = 0.7132061;
	versor[2] = -0.6515190;
	
	eulerConverted = convertVersorToEuler(versor);
	RotationType versorConverted = convertEulerToVersor(eulerConverted);
	
	CPPUNIT_ASSERT( fabs(versor[0] - versorConverted[0]) <= VERSOR_TOLERANCE);
	CPPUNIT_ASSERT( fabs(versor[1] - versorConverted[1]) <= VERSOR_TOLERANCE);
	CPPUNIT_ASSERT( fabs(versor[2] - versorConverted[2]) <= VERSOR_TOLERANCE);
	
}

void CoreTest::fixedWindowRegionTest()
{
	char* argv[6];  	
	argv[0] = (char *) malloc(25);
	strncpy(argv[0], "petspectfusionTest", 25); 
	argv[1] = (char *) malloc(25);
	strncpy(argv[1], "-fregion", 25); 
	argv[2] = (char *) malloc(25);
	strncpy(argv[2], "30", 25); 
	argv[3] = (char *) malloc(25);
	strncpy(argv[3], "40", 25);
	argv[4] = (char *) malloc(25);
	strncpy(argv[4], "50", 25);
	argv[5] = (char *) malloc(25);
	strncpy(argv[5], "20", 25); 
	
	using namespace PetSpectFusionApp;
	
	PetSpectFusionApp::processCommandLine(5, argv);
		
	CPPUNIT_ASSERT( fixedWindowRegion.GetIndex(0) == 10);
	CPPUNIT_ASSERT( fixedWindowRegion.GetIndex(1) == 20);
	CPPUNIT_ASSERT( fixedWindowRegion.GetIndex(2) == 30);
	CPPUNIT_ASSERT( fixedWindowRegion.GetSize(0) == 40);
	CPPUNIT_ASSERT( fixedWindowRegion.GetSize(1) == 40);
	CPPUNIT_ASSERT( fixedWindowRegion.GetSize(2) == 40);
	
}


